@@ -93,6 +93,8 @@ finalize(GObject *gobject)
 
     if (self->node)
         g_object_unref(self->node);
+
+    G_OBJECT_CLASS(view_helper_parent_class)->finalize(gobject);
 }
 
 /* Transform a rectangle from model to view coordinates. */
@@ -138,7 +140,7 @@ update_autoscale(ViewHelper *self)
         if (bbox.width > real_viewport_width || bbox.height > real_viewport_height) {
             float width_ratio = bbox.width / (float)real_viewport_width;
             float height_ratio = bbox.height / (float)real_viewport_height;
-            float max_ratio = width_ratio >= height_ratio ? width_ratio : height_ratio;
+            float max_ratio =  MAX (height_ratio, width_ratio);
 
             zoom_scaled = 1.0 / max_ratio;
 
