@@ -2,7 +2,7 @@
  * Photos - access, organize and share your photos on GNOME
  * Copyright © 2015 Alessandro Bono
  * Copyright © 2014, 2015 Pranav Kant
- * Copyright © 2012, 2013, 2014, 2015 Red Hat, Inc.
+ * Copyright © 2012, 2013, 2014, 2015, 2016 Red Hat, Inc.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -28,6 +28,7 @@
 #include "config.h"
 
 #include <gegl.h>
+#include <gexiv2/gexiv2.h>
 #include <gio/gio.h>
 #include <glib.h>
 #include <glib/gi18n.h>
@@ -38,6 +39,7 @@
 #include "photos-base-item.h"
 #include "photos-camera-cache.h"
 #include "photos-dlna-renderers-dialog.h"
+#include "photos-export-dialog.h"
 #include "photos-filterable.h"
 #include "photos-gom-miner.h"
 #include "photos-item-manager.h"
@@ -69,7 +71,9 @@ struct _PhotosApplicationPrivate
   GSimpleAction *brightness_contrast_action;
   GSimpleAction *crop_action;
   GSimpleAction *denoise_action;
+  GSimpleAction *edit_action;
   GSimpleAction *edit_cancel_action;
+  GSimpleAction *edit_done_action;
   GSimpleAction *fs_action;
   GSimpleAction *gear_action;
   GSimpleAction *insta_action;
@@ -78,6 +82,7 @@ struct _PhotosApplicationPrivate
   GSimpleAction *open_action;
   GSimpleAction *print_action;
   GSimpleAction *properties_action;
+  GSimpleAction *saturation_action;
   GSimpleAction *save_action;
   GSimpleAction *search_action;
   GSimpleAction *search_match_action;
@@ -122,6 +127,26 @@ enum
   MINER_REFRESH_TIMEOUT = 60 /* s */
 };
 
+static const gchar *REQUIRED_GEGL_OPS[] =
+{
+  "gegl:buffer-sink",
+  "gegl:buffer-source",
+  "gegl:crop",
+  "gegl:gray",
+  "gegl:load",
+  "gegl:nop",
+  "gegl:rotate-on-center",
+  "gegl:save-pixbuf",
+  "gegl:scale-ratio",
+
+  /* Used by gegl:load */
+  "gegl:jp2-load",
+  "gegl:jpg-load",
+  "gegl:png-load",
+  "gegl:raw-load",
+  "gegl:text"
+};
+
 static const gchar *DESKTOP_BACKGROUND_SCHEMA = "org.gnome.desktop.background";
 static const gchar *DESKTOP_SCREENSAVER_SCHEMA = "org.gnome.desktop.screensaver";
 static const gchar *DESKTOP_KEY_PICTURE_URI = "picture-uri";
@@ -234,6 +259,64 @@ photos_application_action_toggle (GSimpleAction *simple, GVariant *parameter, gp
 }
 
 
+static void
+photos_application_actions_update (PhotosApplication *self)
+{
+  PhotosApplicationPrivate *priv = self->priv;
+  PhotosBaseItem *item;
+  PhotosLoadState load_state;
+  PhotosWindowMode mode;
+  gboolean enable;
+
+  item = PHOTOS_BASE_ITEM (photos_base_manager_get_active_object (priv->state->item_mngr));
+  load_state = photos_item_manager_get_load_state (priv->state->item_mngr);
+  mode = photos_mode_controller_get_window_mode (priv->state->mode_cntrlr);
+
+  enable = (mode == PHOTOS_WINDOW_MODE_EDIT);
+  g_simple_action_set_enabled (priv->brightness_contrast_action, enable);
+  g_simple_action_set_enabled (priv->crop_action, enable);
+  g_simple_action_set_enabled (priv->denoise_action, enable);
+  g_simple_action_set_enabled (priv->edit_cancel_action, enable);
+  g_simple_action_set_enabled (priv->edit_done_action, enable);
+  g_simple_action_set_enabled (priv->insta_action, enable);
+  g_simple_action_set_enabled (priv->saturation_action, enable);
+  g_simple_action_set_enabled (priv->sharpen_action, enable);
+  g_simple_action_set_enabled (priv->undo_action, enable);
+
+  enable = (mode == PHOTOS_WINDOW_MODE_COLLECTIONS
+            || mode == PHOTOS_WINDOW_MODE_FAVORITES
+            || mode == PHOTOS_WINDOW_MODE_OVERVIEW
+            || mode == PHOTOS_WINDOW_MODE_SEARCH);
+  g_simple_action_set_enabled (priv->search_match_action, enable);
+  g_simple_action_set_enabled (priv->search_source_action, enable);
+  g_simple_action_set_enabled (priv->search_type_action, enable);
+
+  enable = (mode == PHOTOS_WINDOW_MODE_OVERVIEW
+            || mode == PHOTOS_WINDOW_MODE_COLLECTIONS
+            || mode == PHOTOS_WINDOW_MODE_FAVORITES);
+  g_simple_action_set_enabled (priv->sel_all_action, enable);
+  g_simple_action_set_enabled (priv->sel_none_action, enable);
+
+  enable = (mode == PHOTOS_WINDOW_MODE_PREVIEW);
+  g_simple_action_set_enabled (priv->load_next_action, enable);
+  g_simple_action_set_enabled (priv->load_previous_action, enable);
+
+  enable = (load_state == PHOTOS_LOAD_STATE_FINISHED && mode == PHOTOS_WINDOW_MODE_PREVIEW);
+  g_simple_action_set_enabled (priv->gear_action, enable);
+  g_simple_action_set_enabled (priv->open_action, enable);
+  g_simple_action_set_enabled (priv->print_action, enable);
+  g_simple_action_set_enabled (priv->properties_action, enable);
+  g_simple_action_set_enabled (priv->save_action, enable);
+  g_simple_action_set_enabled (priv->set_bg_action, enable);
+  g_simple_action_set_enabled (priv->set_ss_action, enable);
+
+  enable = (load_state == PHOTOS_LOAD_STATE_FINISHED
+            && mode == PHOTOS_WINDOW_MODE_PREVIEW
+            && photos_base_item_can_edit (item));
+  g_simple_action_set_enabled (priv->edit_action, enable);
+}
+
+
 static void
 photos_application_tracker_clear_rdf_types (GObject *source_object, GAsyncResult *res, gpointer user_data)
 {
@@ -368,6 +451,26 @@ photos_application_create_miners (PhotosApplication *self)
 }
 
 
+static gboolean
+photos_application_sanity_check_gegl (PhotosApplication *self)
+{
+  gboolean ret_val = TRUE;
+  guint i;
+
+  for (i = 0; i < G_N_ELEMENTS (REQUIRED_GEGL_OPS); i++)
+    {
+      if (!gegl_has_operation (REQUIRED_GEGL_OPS[i]))
+        {
+          g_warning ("Unable to find GEGL operation %s: Check your GEGL install", REQUIRED_GEGL_OPS[i]);
+          ret_val = FALSE;
+          break;
+        }
+    }
+
+  return ret_val;
+}
+
+
 static void
 photos_application_tracker_set_rdf_types (GObject *source_object, GAsyncResult *res, gpointer user_data)
 {
@@ -419,13 +522,19 @@ photos_application_tracker_extract_priority (GObject *source_object, GAsyncResul
 }
 
 
-static void
+static gboolean
 photos_application_create_window (PhotosApplication *self)
 {
   PhotosApplicationPrivate *priv = self->priv;
+  gboolean gexiv2_initialized;
 
   if (priv->main_window != NULL)
-    return;
+    return TRUE;
+
+  gexiv2_initialized = gexiv2_initialize ();
+  g_return_val_if_fail (gexiv2_initialized, FALSE);
+
+  g_return_val_if_fail (photos_application_sanity_check_gegl (self), FALSE);
 
   priv->main_window = photos_main_window_new (GTK_APPLICATION (self));
   g_signal_connect_swapped (priv->main_window, "destroy", G_CALLBACK (photos_application_destroy), self);
@@ -440,6 +549,7 @@ photos_application_create_window (PhotosApplication *self)
                                               self);
 
   photos_application_start_miners (self);
+  return TRUE;
 }
 
 
@@ -448,7 +558,9 @@ photos_application_activate_item (PhotosApplication *self, GObject *item)
 {
   PhotosApplicationPrivate *priv = self->priv;
 
-  photos_application_create_window (self);
+  if (!photos_application_create_window (self))
+    return;
+
   photos_base_manager_set_active_object (priv->state->item_mngr, item);
   g_application_activate (G_APPLICATION (self));
 
@@ -527,8 +639,7 @@ photos_application_edit_cancel_process (GObject *source_object, GAsyncResult *re
   GError *error = NULL;
   PhotosBaseItem *item = PHOTOS_BASE_ITEM (source_object);
 
-  photos_base_item_process_finish (item, res, &error);
-  if (error != NULL)
+  if (!photos_base_item_process_finish (item, res, &error))
     {
       g_warning ("Unable to process item: %s", error->message);
       g_error_free (error);
@@ -557,6 +668,42 @@ photos_application_edit_cancel (PhotosApplication *self)
 }
 
 
+static void
+photos_application_edit_done_pipeline_save (GObject *source_object, GAsyncResult *res, gpointer user_data)
+{
+  PhotosApplication *self = PHOTOS_APPLICATION (user_data);
+  PhotosBaseItem *item = PHOTOS_BASE_ITEM (source_object);
+  GError *error;
+
+  error = NULL;
+  if (!photos_base_item_pipeline_save_finish (item, res, &error))
+    {
+      g_warning ("Unable to save pipeline: %s", error->message);
+      g_error_free (error);
+      goto out;
+    }
+
+  photos_mode_controller_go_back (self->priv->state->mode_cntrlr);
+
+ out:
+  g_application_release (G_APPLICATION (self));
+}
+
+
+static void
+photos_application_edit_done (PhotosApplication *self)
+{
+  PhotosApplicationPrivate *priv = self->priv;
+  PhotosBaseItem *item;
+
+  item = PHOTOS_BASE_ITEM (photos_base_manager_get_active_object (priv->state->item_mngr));
+  g_return_if_fail (item != NULL);
+
+  g_application_hold (G_APPLICATION (self));
+  photos_base_item_pipeline_save_async (item, NULL, photos_application_edit_done_pipeline_save, self);
+}
+
+
 static void
 photos_application_edit_current (PhotosApplication *self)
 {
@@ -595,7 +742,9 @@ photos_application_launch_search (PhotosApplication *self, const gchar* const *t
   GVariant *state;
   gchar *str;
 
-  photos_application_create_window (self);
+  if (!photos_application_create_window (self))
+    return;
+
   photos_mode_controller_set_window_mode (priv->state->mode_cntrlr, PHOTOS_WINDOW_MODE_OVERVIEW);
 
   str = g_strjoinv (" ", (gchar **) terms);
@@ -610,6 +759,13 @@ photos_application_launch_search (PhotosApplication *self, const gchar* const *t
 }
 
 
+static void
+photos_application_load_changed (PhotosApplication *self)
+{
+  photos_application_actions_update (self);
+}
+
+
 static void
 photos_application_open_current (PhotosApplication *self)
 {
@@ -791,80 +947,91 @@ photos_application_save_save (GObject *source_object, GAsyncResult *res, gpointe
       goto out;
     }
 
-  photos_mode_controller_go_back (self->priv->state->mode_cntrlr);
-
  out:
   g_application_release (G_APPLICATION (self));
 }
 
 
+static void
+photos_application_save_response (GtkDialog *dialog, gint response_id, gpointer user_data)
+{
+  PhotosApplication *self = PHOTOS_APPLICATION (user_data);
+  PhotosApplicationPrivate *priv = self->priv;
+  GError *error;
+  GFile *export = NULL;
+  GFile *tmp;
+  PhotosBaseItem *item;
+  const gchar *export_dir_name;
+  const gchar *pictures_path;
+  gchar *export_path = NULL;
+  gdouble zoom;
+
+  if (response_id != GTK_RESPONSE_OK)
+    goto out;
+
+  item = PHOTOS_BASE_ITEM (photos_base_manager_get_active_object (priv->state->item_mngr));
+  g_return_if_fail (item != NULL);
+
+  pictures_path = g_get_user_special_dir (G_USER_DIRECTORY_PICTURES);
+  export_path = g_build_filename (pictures_path, PHOTOS_EXPORT_SUBPATH, NULL);
+  export = g_file_new_for_path (export_path);
+
+  error = NULL;
+  if (!photos_utils_make_directory_with_parents (export, NULL, &error))
+    {
+      g_warning ("Unable to create %s: %s", export_path, error->message);
+      g_error_free (error);
+      goto out;
+    }
+
+  export_dir_name = photos_export_dialog_get_dir_name (PHOTOS_EXPORT_DIALOG (dialog));
+
+  error = NULL;
+  tmp = g_file_get_child_for_display_name (export, export_dir_name, &error);
+  if (error != NULL)
+    {
+      g_warning ("Unable to get a child for %s: %s", export_dir_name, error->message);
+      g_error_free (error);
+      goto out;
+    }
+
+  g_object_unref (export);
+  export = tmp;
+
+  error = NULL;
+  if (!photos_utils_make_directory_with_parents (export, NULL, &error))
+    {
+      g_warning ("Unable to create %s: %s", export_path, error->message);
+      g_error_free (error);
+      goto out;
+    }
+
+  zoom = photos_export_dialog_get_zoom (PHOTOS_EXPORT_DIALOG (dialog));
+
+  g_application_hold (G_APPLICATION (self));
+  photos_base_item_save_async (item, export, zoom, NULL, photos_application_save_save, self);
+
+ out:
+  g_free (export_path);
+  g_clear_object (&export);
+  gtk_widget_destroy (GTK_WIDGET (dialog));
+}
+
+
 static void
 photos_application_save (PhotosApplication *self)
 {
   PhotosApplicationPrivate *priv = self->priv;
-  GDateTime *now = NULL;
-  GError *error = NULL;
-  GFile *parent = NULL;
+  GtkWidget *dialog;
   PhotosBaseItem *item;
-  const gchar *mime_type;
-  const gchar *pictures_path;
-  gchar *filename = NULL;
-  gchar *extension = NULL;
-  gchar *origin = NULL;
-  gchar *parent_path = NULL;
-  gchar *path = NULL;
-  gchar *type = NULL;
-  gchar *uri = NULL;
 
   item = PHOTOS_BASE_ITEM (photos_base_manager_get_active_object (priv->state->item_mngr));
   g_return_if_fail (item != NULL);
+  g_return_if_fail (!photos_base_item_is_collection (item));
 
-  pictures_path = g_get_user_special_dir (G_USER_DIRECTORY_PICTURES);
-  parent_path = g_build_filename (pictures_path, PACKAGE_TARNAME "-exported", NULL);
-  parent = g_file_new_for_path (parent_path);
-  g_file_make_directory_with_parents (parent, NULL, &error);
-  if (error != NULL)
-    {
-      if (g_error_matches (error, G_IO_ERROR, G_IO_ERROR_EXISTS))
-        {
-          g_error_free (error);
-          goto carry_on;
-        }
-      else
-        {
-          g_warning ("Unable to create %s: %s", parent_path, error->message);
-          g_error_free (error);
-          goto out;
-        }
-    }
-
- carry_on:
-  now = g_date_time_new_now_local ();
-  origin = g_date_time_format (now, "%Y-%m-%d %H-%M-%S");
-
-  mime_type = photos_base_item_get_mime_type (item);
-  extension = photos_utils_get_extension_from_mime_type (mime_type);
-
-  filename = g_strdup_printf ("%s.%s", origin, extension);
-  path = g_build_filename (parent_path, filename, NULL);
-  uri = photos_utils_convert_path_to_uri (path);
-
-  type = photos_utils_get_pixbuf_type_from_mime_type (mime_type);
-
-  g_application_hold (G_APPLICATION (self));
-  photos_base_item_save_async (item, uri, type, NULL, photos_application_save_save, self);
-
- out:
-  if (now != NULL)
-    g_date_time_unref (now);
-
-  g_free (filename);
-  g_free (origin);
-  g_free (parent_path);
-  g_free (path);
-  g_free (type);
-  g_free (uri);
-  g_clear_object (&parent);
+  dialog = photos_export_dialog_new (GTK_WINDOW (priv->main_window), item);
+  gtk_widget_show_all (dialog);
+  g_signal_connect (dialog, "response", G_CALLBACK (photos_application_save_response), self);
 }
 
 
@@ -1012,44 +1179,9 @@ photos_application_theme_changed (GtkSettings *settings)
 
 
 static void
-photos_application_window_mode_changed (PhotosApplication *self, PhotosWindowMode mode, PhotosWindowMode old_mode)
+photos_application_window_mode_changed (PhotosApplication *self)
 {
-  PhotosApplicationPrivate *priv = self->priv;
-  gboolean enable;
-
-  enable = (mode == PHOTOS_WINDOW_MODE_EDIT);
-  g_simple_action_set_enabled (priv->brightness_contrast_action, enable);
-  g_simple_action_set_enabled (priv->crop_action, enable);
-  g_simple_action_set_enabled (priv->denoise_action, enable);
-  g_simple_action_set_enabled (priv->edit_cancel_action, enable);
-  g_simple_action_set_enabled (priv->insta_action, enable);
-  g_simple_action_set_enabled (priv->save_action, enable);
-  g_simple_action_set_enabled (priv->sharpen_action, enable);
-  g_simple_action_set_enabled (priv->undo_action, enable);
-
-  enable = (mode == PHOTOS_WINDOW_MODE_COLLECTIONS
-            || mode == PHOTOS_WINDOW_MODE_FAVORITES
-            || mode == PHOTOS_WINDOW_MODE_OVERVIEW
-            || mode == PHOTOS_WINDOW_MODE_SEARCH);
-  g_simple_action_set_enabled (priv->search_match_action, enable);
-  g_simple_action_set_enabled (priv->search_source_action, enable);
-  g_simple_action_set_enabled (priv->search_type_action, enable);
-
-  enable = (mode == PHOTOS_WINDOW_MODE_OVERVIEW
-            || mode == PHOTOS_WINDOW_MODE_COLLECTIONS
-            || mode == PHOTOS_WINDOW_MODE_FAVORITES);
-  g_simple_action_set_enabled (priv->sel_all_action, enable);
-  g_simple_action_set_enabled (priv->sel_none_action, enable);
-
-  enable = (mode == PHOTOS_WINDOW_MODE_PREVIEW);
-  g_simple_action_set_enabled (priv->gear_action, enable);
-  g_simple_action_set_enabled (priv->load_next_action, enable);
-  g_simple_action_set_enabled (priv->load_previous_action, enable);
-  g_simple_action_set_enabled (priv->open_action, enable);
-  g_simple_action_set_enabled (priv->print_action, enable);
-  g_simple_action_set_enabled (priv->properties_action, enable);
-  g_simple_action_set_enabled (priv->set_bg_action, enable);
-  g_simple_action_set_enabled (priv->set_ss_action, enable);
+  photos_application_actions_update (self);
 }
 
 
@@ -1061,7 +1193,9 @@ photos_application_activate (GApplication *application)
 
   if (priv->main_window == NULL)
     {
-      photos_application_create_window (self);
+      if (!photos_application_create_window (self))
+        return;
+
       photos_mode_controller_set_window_mode (priv->state->mode_cntrlr, PHOTOS_WINDOW_MODE_OVERVIEW);
     }
 
@@ -1135,7 +1269,6 @@ photos_application_shutdown (GApplication *application)
   g_assert (refresh_miner_ids_size == 0);
 
   g_clear_pointer (&priv->refresh_miner_ids, (GDestroyNotify) g_hash_table_unref);
-  gegl_exit ();
 
   G_APPLICATION_CLASS (photos_application_parent_class)->shutdown (application);
 }
@@ -1167,12 +1300,22 @@ photos_application_startup (GApplication *application)
 
   grl_init (NULL, NULL);
   registry = grl_registry_get_default ();
+
   error = NULL;
-  if (!grl_registry_load_plugin_by_id (registry, "grl-flickr", &error))
+  if (!grl_registry_load_all_plugins (registry, FALSE, &error))
     {
-      g_warning ("Unable to load Grilo's Flickr plugin: %s", error->message);
+      g_warning ("Unable to load Grilo plugins: %s", error->message);
       g_error_free (error);
     }
+  else
+    {
+      error = NULL;
+      if (!grl_registry_activate_plugin_by_id (registry, "grl-flickr", &error))
+        {
+          g_warning ("Unable to activate Grilo's Flickr plugin: %s", error->message);
+          g_error_free (error);
+        }
+    }
 
   priv->create_window_cancellable = g_cancellable_new ();
   priv->refresh_miner_ids = g_hash_table_new (g_direct_hash, g_direct_equal);
@@ -1225,10 +1368,16 @@ photos_application_startup (GApplication *application)
                             self);
   g_action_map_add_action (G_ACTION_MAP (self), G_ACTION (priv->edit_cancel_action));
 
-  action = g_simple_action_new ("edit-current", NULL);
-  g_signal_connect_swapped (action, "activate", G_CALLBACK (photos_application_edit_current), self);
-  g_action_map_add_action (G_ACTION_MAP (self), G_ACTION (action));
-  g_object_unref (action);
+  priv->edit_action = g_simple_action_new ("edit-current", NULL);
+  g_signal_connect_swapped (priv->edit_action, "activate", G_CALLBACK (photos_application_edit_current), self);
+  g_action_map_add_action (G_ACTION_MAP (self), G_ACTION (priv->edit_action));
+
+  priv->edit_done_action = g_simple_action_new ("edit-done", NULL);
+  g_signal_connect_swapped (priv->edit_done_action,
+                            "activate",
+                            G_CALLBACK (photos_application_edit_done),
+                            self);
+  g_action_map_add_action (G_ACTION_MAP (self), G_ACTION (priv->edit_done_action));
 
   priv->fs_action = g_simple_action_new ("fullscreen", NULL);
   g_signal_connect_swapped (priv->fs_action, "activate", G_CALLBACK (photos_application_fullscreen), self);
@@ -1275,6 +1424,9 @@ photos_application_startup (GApplication *application)
   g_action_map_add_action (G_ACTION_MAP (self), G_ACTION (action));
   g_object_unref (action);
 
+  priv->saturation_action = g_simple_action_new ("saturation-current", G_VARIANT_TYPE_DOUBLE);
+  g_action_map_add_action (G_ACTION_MAP (self), G_ACTION (priv->saturation_action));
+
   priv->save_action = g_simple_action_new ("save-current", NULL);
   g_signal_connect_swapped (priv->save_action, "activate", G_CALLBACK (photos_application_save), self);
   g_action_map_add_action (G_ACTION_MAP (self), G_ACTION (priv->save_action));
@@ -1347,6 +1499,15 @@ photos_application_startup (GApplication *application)
   gtk_application_set_accels_for_action (GTK_APPLICATION (self), "app.print-current", print_current_accels);
   gtk_application_set_accels_for_action (GTK_APPLICATION (self), "app.search", search_accels);
   gtk_application_set_accels_for_action (GTK_APPLICATION (self), "app.select-all", select_all_accels);
+
+  g_signal_connect_swapped (priv->state->item_mngr,
+                            "load-finished",
+                            G_CALLBACK (photos_application_load_changed),
+                            self);
+  g_signal_connect_swapped (priv->state->item_mngr,
+                            "load-started",
+                            G_CALLBACK (photos_application_load_changed),
+                            self);
 }
 
 
@@ -1398,7 +1559,9 @@ photos_application_dispose (GObject *object)
   g_clear_object (&priv->brightness_contrast_action);
   g_clear_object (&priv->crop_action);
   g_clear_object (&priv->denoise_action);
+  g_clear_object (&priv->edit_action);
   g_clear_object (&priv->edit_cancel_action);
+  g_clear_object (&priv->edit_done_action);
   g_clear_object (&priv->fs_action);
   g_clear_object (&priv->gear_action);
   g_clear_object (&priv->insta_action);
@@ -1407,6 +1570,7 @@ photos_application_dispose (GObject *object)
   g_clear_object (&priv->open_action);
   g_clear_object (&priv->print_action);
   g_clear_object (&priv->properties_action);
+  g_clear_object (&priv->saturation_action);
   g_clear_object (&priv->save_action);
   g_clear_object (&priv->search_action);
   g_clear_object (&priv->search_match_action);
@@ -1438,6 +1602,9 @@ photos_application_finalize (GObject *object)
 
   g_assert (self->priv->create_miners_count == 0);
 
+  if (!g_application_get_is_remote (G_APPLICATION (self)))
+    gegl_exit ();
+
   G_OBJECT_CLASS (photos_application_parent_class)->finalize (object);
 }
 
