@@ -1,7 +1,8 @@
 /*
  * Photos - access, organize and share your photos on GNOME
  * Copyright © 2014, 2015 Pranav Kant
- * Copyright © 2012, 2013, 2014, 2015 Red Hat, Inc.
+ * Copyright © 2012, 2013, 2014, 2015, 2016 Red Hat, Inc.
+ * Copyright © 2016 Umang Jain
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -31,6 +32,7 @@
 
 #include <gdk/gdk.h>
 #include <gegl-plugin.h>
+#include <gexiv2/gexiv2.h>
 #include <gio/gio.h>
 #include <glib.h>
 #include <glib/gi18n.h>
@@ -126,6 +128,15 @@ G_DEFINE_ABSTRACT_TYPE_WITH_CODE (PhotosBaseItem, photos_base_item, G_TYPE_OBJEC
                                                          photos_base_item_filterable_iface_init));
 
 
+typedef struct _PhotosBaseItemSaveData PhotosBaseItemSaveData;
+
+struct _PhotosBaseItemSaveData
+{
+  GFile *dir;
+  GeglBuffer *buffer;
+  gchar *type;
+};
+
 static GdkPixbuf *failed_icon;
 static GdkPixbuf *thumbnailing_icon;
 static GThreadPool *create_thumbnail_pool;
@@ -134,6 +145,35 @@ static GThreadPool *create_thumbnail_pool;
 static void photos_base_item_populate_from_cursor (PhotosBaseItem *self, TrackerSparqlCursor *cursor);
 
 
+static PhotosBaseItemSaveData *
+photos_base_item_save_data_new (GFile *dir, GeglBuffer *buffer, const gchar *type)
+{
+  PhotosBaseItemSaveData *data;
+
+  data = g_slice_new0 (PhotosBaseItemSaveData);
+
+  if (dir != NULL)
+    data->dir = g_object_ref (dir);
+
+  if (buffer != NULL)
+    data->buffer = g_object_ref (buffer);
+
+  data->type = g_strdup (type);
+
+  return data;
+}
+
+
+static void
+photos_base_item_save_data_free (PhotosBaseItemSaveData *data)
+{
+  g_clear_object (&data->dir);
+  g_clear_object (&data->buffer);
+  g_free (data->type);
+  g_slice_free (PhotosBaseItemSaveData, data);
+}
+
+
 static GdkPixbuf *
 photos_base_item_create_placeholder_icon (const gchar *icon_name)
 {
@@ -779,10 +819,11 @@ photos_base_item_load_buffer_async (PhotosBaseItem *self,
                                     GAsyncReadyCallback callback,
                                     gpointer user_data)
 {
-  PhotosBaseItemPrivate *priv = self->priv;
+  PhotosBaseItemPrivate *priv;
   GTask *task;
 
   g_return_if_fail (PHOTOS_IS_BASE_ITEM (self));
+  priv = self->priv;
 
   if (priv->load_graph == NULL)
     {
@@ -795,17 +836,6 @@ photos_base_item_load_buffer_async (PhotosBaseItem *self,
       gegl_node_link_many (priv->load, orientation, priv->buffer_sink, NULL);
     }
 
-  if (priv->edit_graph == NULL)
-    {
-      GeglNode *graph;
-
-      priv->edit_graph = gegl_node_new ();
-      priv->buffer_source = gegl_node_new_child (priv->edit_graph, "operation", "gegl:buffer-source", NULL);
-      priv->pipeline = photos_pipeline_new (priv->edit_graph);
-      graph = photos_pipeline_get_graph (priv->pipeline);
-      gegl_node_link (priv->buffer_source, graph);
-    }
-
   task = g_task_new (self, cancellable, callback, user_data);
   g_task_set_source_tag (task, photos_base_item_load_buffer_async);
 
@@ -895,48 +925,274 @@ photos_base_item_load_load_buffer (GObject *source_object, GAsyncResult *res, gp
 }
 
 
+static void
+photos_base_item_load_pipeline (GObject *source_object, GAsyncResult *res, gpointer user_data)
+{
+  GTask *task = G_TASK (user_data);
+  PhotosBaseItem *self;
+  PhotosBaseItemPrivate *priv;
+  GeglNode *graph;
+  GCancellable *cancellable;
+  GError *error;
+
+  self = PHOTOS_BASE_ITEM (g_task_get_source_object (task));
+  priv = self->priv;
+
+  cancellable = g_task_get_cancellable (task);
+
+  error = NULL;
+  priv->pipeline = photos_pipeline_new_finish (res, &error);
+  if (error != NULL)
+    {
+      g_task_return_error (task, error);
+      goto out;
+    }
+
+  graph = photos_pipeline_get_graph (priv->pipeline);
+  gegl_node_link (priv->buffer_source, graph);
+
+  photos_base_item_load_buffer_async (self, cancellable, photos_base_item_load_load_buffer, g_object_ref (task));
+
+ out:
+  g_object_unref (task);
+}
+
+
+static void
+photos_base_item_pipeline_save_save (GObject *source_object, GAsyncResult *res, gpointer user_data)
+{
+  GTask *task = G_TASK (user_data);
+  PhotosBaseItem *self;
+  GError *error;
+
+  self = PHOTOS_BASE_ITEM (g_task_get_source_object (task));
+
+  error = NULL;
+  if (!photos_pipeline_save_finish (self->priv->pipeline, res, &error))
+    {
+      g_task_return_error (task, error);
+      goto out;
+    }
+
+  g_task_return_boolean (task, TRUE);
+
+ out:
+  g_object_unref (task);
+}
+
+
 static gboolean
 photos_base_item_process_idle (gpointer user_data)
 {
   GTask *task = G_TASK (user_data);
   PhotosBaseItem *self;
-  GCancellable *cancellable;
-  gboolean result = FALSE;
 
   self = PHOTOS_BASE_ITEM (g_task_get_source_object (task));
-  cancellable = g_task_get_cancellable (task);
 
-  if (g_cancellable_is_cancelled (cancellable))
+  if (g_task_return_error_if_cancelled (task))
     goto done;
 
   if (gegl_processor_work (self->priv->processor, NULL))
     return G_SOURCE_CONTINUE;
 
-  result = TRUE;
+  g_task_return_boolean (task, TRUE);
 
  done:
-  g_task_return_boolean (task, result);
   return G_SOURCE_REMOVE;
 }
 
 
+static void
+photos_base_item_save_guess_sizes_from_buffer (GeglBuffer *buffer,
+                                               const gchar *mime_type,
+                                               gsize *out_size,
+                                               gsize *out_size_1,
+                                               GCancellable *cancellable)
+{
+  GeglNode *buffer_source;
+  GeglNode *graph;
+  GeglNode *guess_sizes;
+  gsize sizes[2];
+
+  graph = gegl_node_new ();
+  buffer_source = gegl_node_new_child (graph, "operation", "gegl:buffer-source", "buffer", buffer, NULL);
+
+  if (g_strcmp0 (mime_type, "image/png") == 0)
+    guess_sizes = gegl_node_new_child (graph,
+                                       "operation", "photos:png-guess-sizes",
+                                       "background", FALSE,
+                                       "bitdepth", 8,
+                                       "compression", -1,
+                                       NULL);
+  else
+    guess_sizes = gegl_node_new_child (graph,
+                                       "operation", "photos:jpg-guess-sizes",
+                                       "optimize", FALSE,
+                                       "progressive", FALSE,
+                                       "sampling", TRUE,
+                                       NULL);
+
+  gegl_node_link (buffer_source, guess_sizes);
+  gegl_node_process (guess_sizes);
+
+  gegl_node_get (guess_sizes, "size", &sizes[0], "size-1", &sizes[1], NULL);
+  if (out_size != NULL)
+    *out_size = sizes[0];
+  if (out_size_1 != NULL)
+    *out_size_1 = sizes[1];
+
+  g_object_unref (graph);
+}
+
+
+static void
+photos_base_item_save_guess_sizes_in_thread_func (GTask *task,
+                                                  gpointer source_object,
+                                                  gpointer task_data,
+                                                  GCancellable *cancellable)
+{
+  PhotosBaseItemSaveData *data = (PhotosBaseItemSaveData *) task_data;
+  gsize *sizes;
+
+  sizes = g_malloc0_n (2, sizeof (gsize));
+  photos_base_item_save_guess_sizes_from_buffer (data->buffer, data->type, &sizes[0], &sizes[1], cancellable);
+  g_task_return_pointer (task, sizes, g_free);
+}
+
+
+static void
+photos_base_item_save_metadata_in_thread_func (GTask *task,
+                                               gpointer source_object,
+                                               gpointer task_data,
+                                               GCancellable *cancellable)
+{
+  PhotosBaseItem *self = PHOTOS_BASE_ITEM (source_object);
+  PhotosBaseItemPrivate *priv = self->priv;
+  GError *error;
+  GFile *file = G_FILE (task_data);
+  GExiv2Metadata *metadata = NULL;
+  gchar *export_path = NULL;
+  gchar *source_path = NULL;
+
+  g_mutex_lock (&priv->mutex_download);
+
+  error = NULL;
+  source_path = photos_base_item_download (self, cancellable, &error);
+  if (error != NULL)
+    {
+      g_task_return_error (task, error);
+      goto out;
+    }
+
+  metadata = gexiv2_metadata_new ();
+
+  error = NULL;
+  if (!gexiv2_metadata_open_path (metadata, source_path, &error))
+    {
+      g_task_return_error (task, error);
+      goto out;
+    }
+
+  export_path = g_file_get_path (file);
+
+  error = NULL;
+  if (!gexiv2_metadata_save_file (metadata, export_path, &error))
+    {
+      g_task_return_error (task, error);
+      goto out;
+    }
+
+  g_task_return_boolean (task, TRUE);
+
+ out:
+  g_mutex_unlock (&priv->mutex_download);
+  g_clear_object (&metadata);
+  g_free (export_path);
+  g_free (source_path);
+}
+
+
+static void
+photos_base_item_save_metadata_async (PhotosBaseItem *self,
+                                      GFile *file,
+                                      GCancellable *cancellable,
+                                      GAsyncReadyCallback callback,
+                                      gpointer user_data)
+{
+  GTask *task;
+
+  task = g_task_new (self, cancellable, callback, user_data);
+  g_task_set_source_tag (task, photos_base_item_save_metadata_async);
+  g_task_set_task_data (task, g_object_ref (file), g_object_unref);
+
+  g_task_run_in_thread (task, photos_base_item_save_metadata_in_thread_func);
+  g_object_unref (task);
+}
+
+
+static gboolean
+photos_base_item_save_metadata_finish (PhotosBaseItem *self, GAsyncResult *res, GError **error)
+{
+  GTask *task = G_TASK (res);
+
+  g_return_val_if_fail (g_task_is_valid (res, self), FALSE);
+  g_return_val_if_fail (g_task_get_source_tag (task) == photos_base_item_save_metadata_async, FALSE);
+  g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
+
+  return g_task_propagate_boolean (task, error);
+}
+
+
+static void
+photos_base_item_save_save_metadata (GObject *source_object, GAsyncResult *res, gpointer user_data)
+{
+  PhotosBaseItem *self = PHOTOS_BASE_ITEM (source_object);
+  GError *error;
+  GTask *task = G_TASK (user_data);
+
+  error = NULL;
+  if (!photos_base_item_save_metadata_finish (self, res, &error))
+    {
+      g_task_return_error (task, error);
+      goto out;
+    }
+
+  g_task_return_boolean (task, TRUE);
+
+ out:
+  g_object_unref (task);
+}
+
+
 static void
 photos_base_item_save_save_to_stream (GObject *source_object, GAsyncResult *res, gpointer user_data)
 {
   GTask *task = G_TASK (user_data);
+  PhotosBaseItem *self;
+  PhotosBaseItemSaveData *data;
+  GCancellable *cancellable;
   GError *error = NULL;
-  gboolean result;
+  GFile *file = NULL;
 
-  result = gdk_pixbuf_save_to_stream_finish (res, &error);
-  if (error != NULL)
+  self = PHOTOS_BASE_ITEM (g_task_get_source_object (task));
+  cancellable = g_task_get_cancellable (task);
+  data = (PhotosBaseItemSaveData *) g_task_get_task_data (task);
+
+  if (!gdk_pixbuf_save_to_stream_finish (res, &error))
     {
       g_task_return_error (task, error);
       goto out;
     }
 
-  g_task_return_boolean (task, result);
+  file = g_file_get_child (data->dir, self->priv->filename);
+  photos_base_item_save_metadata_async (self,
+                                        file,
+                                        cancellable,
+                                        photos_base_item_save_save_metadata,
+                                        g_object_ref (task));
 
  out:
+  g_clear_object (&file);
   g_object_unref (task);
 }
 
@@ -945,21 +1201,17 @@ static void
 photos_base_item_save_replace (GObject *source_object, GAsyncResult *res, gpointer user_data)
 {
   GTask *task = G_TASK (user_data);
-  PhotosBaseItem *self;
-  PhotosBaseItemPrivate *priv;
   GCancellable *cancellable;
   GdkPixbuf *pixbuf = NULL;
   GError *error = NULL;
-  GeglNode *graph;
+  GeglNode *buffer_source;
+  GeglNode *graph = NULL;
   GFile *file = G_FILE (source_object);
   GFileOutputStream *stream = NULL;
-  const gchar *type;
-
-  self = PHOTOS_BASE_ITEM (g_task_get_source_object (task));
-  priv = self->priv;
+  PhotosBaseItemSaveData *data;
 
   cancellable = g_task_get_cancellable (task);
-  type = (const gchar *) g_task_get_task_data (task);
+  data = (PhotosBaseItemSaveData *) g_task_get_task_data (task);
 
   stream = g_file_replace_finish (file, res, &error);
   if (error != NULL)
@@ -968,41 +1220,89 @@ photos_base_item_save_replace (GObject *source_object, GAsyncResult *res, gpoint
       goto out;
     }
 
-  if (priv->processor == NULL)
-    {
-      g_task_return_new_error (task, PHOTOS_ERROR, 0, "Failed to find a processor for the GEGL graph");
-      goto out;
-    }
-
-  if (gegl_processor_work (priv->processor, NULL))
-    {
-      g_task_return_new_error (task, PHOTOS_ERROR, 0, "Have not finished processing the GEGL graph");
-      goto out;
-    }
-
-  graph = photos_pipeline_get_graph (priv->pipeline);
-  pixbuf = photos_utils_create_pixbuf_from_node (graph);
+  graph = gegl_node_new ();
+  buffer_source = gegl_node_new_child (graph, "operation", "gegl:buffer-source", "buffer", data->buffer, NULL);
+  pixbuf = photos_utils_create_pixbuf_from_node (buffer_source);
   if (pixbuf == NULL)
     {
-      g_task_return_new_error (task, PHOTOS_ERROR, 0, "Failed to create a GdkPixbuf from the GEGL graph");
+      g_task_return_new_error (task, PHOTOS_ERROR, 0, "Failed to create a GdkPixbuf from the GeglBuffer");
       goto out;
     }
 
-  gdk_pixbuf_save_to_stream_async (pixbuf,
-                                   G_OUTPUT_STREAM (stream),
-                                   type,
-                                   cancellable,
-                                   photos_base_item_save_save_to_stream,
-                                   g_object_ref (task),
-                                   NULL);
+  if (g_strcmp0 (data->type, "jpeg") == 0)
+    {
+      gdk_pixbuf_save_to_stream_async (pixbuf,
+                                       G_OUTPUT_STREAM (stream),
+                                       data->type,
+                                       cancellable,
+                                       photos_base_item_save_save_to_stream,
+                                       g_object_ref (task),
+                                       "quality", "90",
+                                       NULL);
+    }
+  else
+    {
+      gdk_pixbuf_save_to_stream_async (pixbuf,
+                                       G_OUTPUT_STREAM (stream),
+                                       data->type,
+                                       cancellable,
+                                       photos_base_item_save_save_to_stream,
+                                       g_object_ref (task),
+                                       NULL);
+    }
 
  out:
+  g_clear_object (&graph);
   g_clear_object (&pixbuf);
   g_clear_object (&stream);
   g_object_unref (task);
 }
 
 
+static void
+photos_base_item_save_buffer_zoom (GObject *source_object, GAsyncResult *res, gpointer user_data)
+{
+  GTask *task = G_TASK (user_data);
+  PhotosBaseItem *self;
+  GCancellable *cancellable;
+  GError *error;
+  GFile *file = NULL;
+  GeglBuffer *buffer = GEGL_BUFFER (source_object);
+  GeglBuffer *buffer_zoomed = NULL;
+  PhotosBaseItemSaveData *data;
+
+  self = PHOTOS_BASE_ITEM (g_task_get_source_object (task));
+  cancellable = g_task_get_cancellable (task);
+  data = (PhotosBaseItemSaveData *) g_task_get_task_data (task);
+
+  error = NULL;
+  buffer_zoomed = photos_utils_buffer_zoom_finish (buffer, res, &error);
+  if (error != NULL)
+    {
+      g_task_return_error (task, error);
+      goto out;
+    }
+
+  g_assert_null (data->buffer);
+  data->buffer = g_object_ref (buffer_zoomed);
+
+  file = g_file_get_child (data->dir, self->priv->filename);
+  g_file_replace_async (file,
+                        NULL,
+                        TRUE,
+                        G_FILE_CREATE_REPLACE_DESTINATION,
+                        G_PRIORITY_DEFAULT,
+                        cancellable,
+                        photos_base_item_save_replace,
+                        g_object_ref (task));
+
+ out:
+  g_clear_object (&buffer_zoomed);
+  g_clear_object (&file);
+  g_object_unref (task);
+}
+
+
 static void
 photos_base_item_set_thumbnailing_icon (PhotosBaseItem *self)
 {
@@ -1359,6 +1659,20 @@ photos_base_item_filterable_iface_init (PhotosFilterableInterface *iface)
 }
 
 
+gboolean
+photos_base_item_can_edit (PhotosBaseItem *self)
+{
+  PhotosBaseItemPrivate *priv;
+
+  g_return_val_if_fail (PHOTOS_IS_BASE_ITEM (self), FALSE);
+  priv = self->priv;
+
+  return PHOTOS_BASE_ITEM_GET_CLASS (self)->create_pipeline_path != NULL
+    && priv->filename != NULL
+    && priv->filename[0] != '\0';
+}
+
+
 gboolean
 photos_base_item_can_trash (PhotosBaseItem *self)
 {
@@ -1375,6 +1689,7 @@ photos_base_item_create_preview (PhotosBaseItem *self,
                                  ...)
 {
   PhotosBaseItemPrivate *priv = self->priv;
+  const Babl *format;
   GeglBuffer *buffer_orig = NULL;
   GeglBuffer *buffer = NULL;
   GeglNode *buffer_source;
@@ -1387,7 +1702,6 @@ photos_base_item_create_preview (PhotosBaseItem *self,
   GeglRectangle roi;
   cairo_surface_t *surface = NULL;
   static const cairo_user_data_key_t key;
-  const Babl *format;
   const gchar *name;
   gdouble x;
   gdouble y;
@@ -1415,7 +1729,7 @@ photos_base_item_create_preview (PhotosBaseItem *self,
   buffer = gegl_buffer_dup (buffer_orig);
 
   bbox = *gegl_buffer_get_extent (buffer);
-  min_dimension = bbox.height < bbox.width ? bbox.height : bbox.width;
+  min_dimension = MIN (bbox.height, bbox.width);
   x = (gdouble) (bbox.width - min_dimension) / 2.0;
   y = (gdouble) (bbox.height - min_dimension) / 2.0;
   size_scaled = size * scale;
@@ -1527,6 +1841,32 @@ photos_base_item_get_author (PhotosBaseItem *self)
 }
 
 
+gboolean
+photos_base_item_get_bbox_edited (PhotosBaseItem *self, GeglRectangle *out_bbox)
+{
+  PhotosBaseItemPrivate *priv;
+  GeglNode *graph;
+  GeglRectangle bbox;
+
+  g_return_val_if_fail (PHOTOS_IS_BASE_ITEM (self), FALSE);
+  priv = self->priv;
+
+  g_return_val_if_fail (priv->edit_graph != NULL, FALSE);
+  g_return_val_if_fail (priv->load_graph != NULL, FALSE);
+  g_return_val_if_fail (priv->pipeline != NULL, FALSE);
+  g_return_val_if_fail (priv->processor != NULL, FALSE);
+  g_return_val_if_fail (!gegl_processor_work (priv->processor, NULL), FALSE);
+
+  graph = photos_pipeline_get_graph (priv->pipeline);
+  bbox = gegl_node_get_bounding_box (graph);
+
+  if (out_bbox != NULL)
+    *out_bbox = bbox;
+
+  return TRUE;
+}
+
+
 gboolean
 photos_base_item_get_bbox_source (PhotosBaseItem *self, GeglRectangle *bbox)
 {
@@ -1740,14 +2080,49 @@ photos_base_item_load_async (PhotosBaseItem *self,
                              GAsyncReadyCallback callback,
                              gpointer user_data)
 {
+  PhotosBaseItemPrivate *priv;
   GTask *task;
 
   g_return_if_fail (PHOTOS_IS_BASE_ITEM (self));
+  priv = self->priv;
+
+  g_return_if_fail (priv->edit_graph == NULL || priv->pipeline != NULL);
 
   task = g_task_new (self, cancellable, callback, user_data);
   g_task_set_source_tag (task, photos_base_item_load_async);
 
-  photos_base_item_load_buffer_async (self, cancellable, photos_base_item_load_load_buffer, g_object_ref (task));
+  if (priv->pipeline != NULL)
+    {
+      GeglNode *graph;
+
+      graph = photos_pipeline_get_graph (priv->pipeline);
+      g_task_return_pointer (task, g_object_ref (graph), g_object_unref);
+    }
+  else
+    {
+      PhotosBaseItemClass *class;
+      gchar *uri = NULL;
+
+      class = PHOTOS_BASE_ITEM_GET_CLASS (self);
+      if (class->create_pipeline_path != NULL)
+        {
+          gchar *path;
+
+          path = class->create_pipeline_path (self);
+          uri = photos_utils_convert_path_to_uri (path);
+          g_free (path);
+        }
+
+      priv->edit_graph = gegl_node_new ();
+      priv->buffer_source = gegl_node_new_child (priv->edit_graph, "operation", "gegl:buffer-source", NULL);
+      photos_pipeline_new_async (priv->edit_graph,
+                                 uri,
+                                 cancellable,
+                                 photos_base_item_load_pipeline,
+                                 g_object_ref (task));
+      g_free (uri);
+    }
+
   g_object_unref (task);
 }
 
@@ -1804,6 +2179,47 @@ photos_base_item_operation_undo (PhotosBaseItem *self)
 }
 
 
+void
+photos_base_item_pipeline_save_async (PhotosBaseItem *self,
+                                      GCancellable *cancellable,
+                                      GAsyncReadyCallback callback,
+                                      gpointer user_data)
+{
+  PhotosBaseItemPrivate *priv;
+  GTask *task;
+
+  g_return_if_fail (PHOTOS_IS_BASE_ITEM (self));
+  priv = self->priv;
+
+  g_return_if_fail (priv->edit_graph != NULL);
+  g_return_if_fail (priv->load_graph != NULL);
+  g_return_if_fail (priv->pipeline != NULL);
+
+  task = g_task_new (self, cancellable, callback, user_data);
+  g_task_set_source_tag (task, photos_base_item_pipeline_save_async);
+
+  photos_pipeline_save_async (priv->pipeline,
+                              cancellable,
+                              photos_base_item_pipeline_save_save,
+                              g_object_ref (task));
+
+  g_object_unref (task);
+}
+
+
+gboolean
+photos_base_item_pipeline_save_finish (PhotosBaseItem *self, GAsyncResult *res, GError **error)
+{
+  GTask *task = G_TASK (res);
+
+  g_return_val_if_fail (g_task_is_valid (res, self), FALSE);
+  g_return_val_if_fail (g_task_get_source_tag (task) == photos_base_item_pipeline_save_async, FALSE);
+  g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
+
+  return g_task_propagate_boolean (task, error);
+}
+
+
 void
 photos_base_item_print (PhotosBaseItem *self, GtkWidget *toplevel)
 {
@@ -1864,39 +2280,50 @@ photos_base_item_refresh (PhotosBaseItem *self)
 
 void
 photos_base_item_save_async (PhotosBaseItem *self,
-                             const gchar *uri,
-                             const gchar *type,
+                             GFile *dir,
+                             gdouble zoom,
                              GCancellable *cancellable,
                              GAsyncReadyCallback callback,
                              gpointer user_data)
 {
-  PhotosBaseItemPrivate *priv = self->priv;
-  GFile *file;
+  PhotosBaseItemPrivate *priv;
   GTask *task;
+  GeglBuffer *buffer;
+  GeglNode *graph;
+  PhotosBaseItemSaveData *data;
+  const gchar *type;
 
   g_return_if_fail (PHOTOS_IS_BASE_ITEM (self));
-  g_return_if_fail (uri != NULL && uri[0] != '\0');
-  g_return_if_fail (type != NULL && type[0] != '\0');
+  priv = self->priv;
+
+  g_return_if_fail (G_IS_FILE (dir));
   g_return_if_fail (priv->edit_graph != NULL);
+  g_return_if_fail (priv->filename != NULL && priv->filename[0] != '\0');
   g_return_if_fail (priv->load_graph != NULL);
+  g_return_if_fail (priv->pipeline != NULL);
   g_return_if_fail (priv->processor != NULL);
   g_return_if_fail (!gegl_processor_work (priv->processor, NULL));
 
+  if (g_strcmp0 (priv->mime_type, "image/png") == 0)
+    type = "png";
+  else
+    type = "jpeg";
+
+  data = photos_base_item_save_data_new (dir, NULL, type);
+
   task = g_task_new (self, cancellable, callback, user_data);
   g_task_set_source_tag (task, photos_base_item_save_async);
-  g_task_set_task_data (task, g_strdup (type), g_free);
+  g_task_set_task_data (task, data, (GDestroyNotify) photos_base_item_save_data_free);
 
-  file = g_file_new_for_uri (uri);
-  g_file_replace_async (file,
-                        NULL,
-                        TRUE,
-                        G_FILE_CREATE_REPLACE_DESTINATION,
-                        G_PRIORITY_DEFAULT,
-                        cancellable,
-                        photos_base_item_save_replace,
-                        g_object_ref (task));
+  graph = photos_pipeline_get_graph (priv->pipeline);
+  buffer = photos_utils_create_buffer_from_node (graph);
+  photos_utils_buffer_zoom_async (buffer,
+                                  zoom,
+                                  cancellable,
+                                  photos_base_item_save_buffer_zoom,
+                                  g_object_ref (task));
 
-  g_object_unref (file);
+  g_object_unref (buffer);
   g_object_unref (task);
 }
 
@@ -1914,6 +2341,74 @@ photos_base_item_save_finish (PhotosBaseItem *self, GAsyncResult *res, GError **
 }
 
 
+void
+photos_base_item_save_guess_sizes_async (PhotosBaseItem *self,
+                                         GCancellable *cancellable,
+                                         GAsyncReadyCallback callback,
+                                         gpointer user_data)
+{
+  PhotosBaseItemPrivate *priv;
+  GeglBuffer *buffer;
+  GeglNode *graph;
+  GTask *task;
+  PhotosBaseItemSaveData *data;
+
+  g_return_if_fail (PHOTOS_IS_BASE_ITEM (self));
+  priv = self->priv;
+
+  g_return_if_fail (priv->edit_graph != NULL);
+  g_return_if_fail (priv->load_graph != NULL);
+  g_return_if_fail (priv->pipeline != NULL);
+  g_return_if_fail (priv->processor != NULL);
+  g_return_if_fail (!gegl_processor_work (priv->processor, NULL));
+
+  graph = photos_pipeline_get_graph (priv->pipeline);
+  buffer = photos_utils_create_buffer_from_node (graph);
+
+  data = photos_base_item_save_data_new (NULL, buffer, priv->mime_type);
+
+  task = g_task_new (self, cancellable, callback, user_data);
+  g_task_set_source_tag (task, photos_base_item_save_guess_sizes_async);
+  g_task_set_task_data (task, data, (GDestroyNotify) photos_base_item_save_data_free);
+
+  g_task_run_in_thread (task, photos_base_item_save_guess_sizes_in_thread_func);
+
+  g_object_unref (buffer);
+  g_object_unref (task);
+}
+
+
+gboolean
+photos_base_item_save_guess_sizes_finish (PhotosBaseItem *self,
+                                          GAsyncResult *res,
+                                          gsize *out_size,
+                                          gsize *out_size_1,
+                                          GError **error)
+{
+  GTask *task = G_TASK (res);
+  gboolean ret_val = FALSE;
+  gsize *sizes;
+
+  g_return_val_if_fail (g_task_is_valid (res, self), FALSE);
+  g_return_val_if_fail (g_task_get_source_tag (task) == photos_base_item_save_guess_sizes_async, FALSE);
+  g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
+
+  sizes = g_task_propagate_pointer (task, error);
+  if (g_task_had_error (task))
+    goto out;
+
+  ret_val = TRUE;
+
+  if (out_size != NULL)
+    *out_size = sizes[0];
+  if (out_size_1 != NULL)
+    *out_size_1 = sizes[1];
+
+ out:
+  return ret_val;
+}
+
+
 void
 photos_base_item_set_default_app (PhotosBaseItem *self, GAppInfo *default_app)
 {
