@@ -75,6 +75,7 @@ struct _PhotosBaseItemClass
 
   /* virtual methods */
   gchar      *(*create_name_fallback)       (PhotosBaseItem *self);
+  gchar      *(*create_pipeline_path)       (PhotosBaseItem *self);
   gboolean    (*create_thumbnail)           (PhotosBaseItem *self, GCancellable *cancellable, GError **error);
   gchar      *(*download)                   (PhotosBaseItem *self, GCancellable *cancellable, GError **error);
   GtkWidget  *(*get_source_widget)          (PhotosBaseItem *self);
@@ -89,6 +90,8 @@ struct _PhotosBaseItemClass
 
 GType               photos_base_item_get_type                (void) G_GNUC_CONST;
 
+gboolean            photos_base_item_can_edit                (PhotosBaseItem *self);
+
 gboolean            photos_base_item_can_trash               (PhotosBaseItem *self);
 
 cairo_surface_t    *photos_base_item_create_preview          (PhotosBaseItem *self,
@@ -115,6 +118,8 @@ gchar              *photos_base_item_download_finish         (PhotosBaseItem *se
 
 const gchar        *photos_base_item_get_author              (PhotosBaseItem *self);
 
+gboolean            photos_base_item_get_bbox_edited         (PhotosBaseItem *self, GeglRectangle *out_bbox);
+
 gboolean            photos_base_item_get_bbox_source         (PhotosBaseItem *self, GeglRectangle *bbox);
 
 gint64              photos_base_item_get_date_created        (PhotosBaseItem *self);
@@ -192,6 +197,15 @@ gboolean            photos_base_item_operation_get           (PhotosBaseItem *se
 
 gboolean            photos_base_item_operation_undo          (PhotosBaseItem *self);
 
+void                photos_base_item_pipeline_save_async     (PhotosBaseItem *self,
+                                                              GCancellable *cancellable,
+                                                              GAsyncReadyCallback callback,
+                                                              gpointer user_data);
+
+gboolean            photos_base_item_pipeline_save_finish    (PhotosBaseItem *self,
+                                                              GAsyncResult *res,
+                                                              GError **error);
+
 void                photos_base_item_print                   (PhotosBaseItem *self, GtkWidget *toplevel);
 
 void                photos_base_item_process_async           (PhotosBaseItem *self,
@@ -206,8 +220,8 @@ gboolean            photos_base_item_process_finish          (PhotosBaseItem *se
 void                photos_base_item_refresh                 (PhotosBaseItem *self);
 
 void                photos_base_item_save_async              (PhotosBaseItem *self,
-                                                              const gchar *uri,
-                                                              const gchar *type,
+                                                              GFile *dir,
+                                                              gdouble zoom,
                                                               GCancellable *cancellable,
                                                               GAsyncReadyCallback callback,
                                                               gpointer user_data);
@@ -216,6 +230,17 @@ gboolean            photos_base_item_save_finish             (PhotosBaseItem *se
                                                               GAsyncResult *res,
                                                               GError **error);
 
+void                photos_base_item_save_guess_sizes_async  (PhotosBaseItem *self,
+                                                              GCancellable *cancellable,
+                                                              GAsyncReadyCallback callback,
+                                                              gpointer user_data);
+
+gboolean            photos_base_item_save_guess_sizes_finish (PhotosBaseItem *self,
+                                                              GAsyncResult *res,
+                                                              gsize *out_size,
+                                                              gsize *out_size_1,
+                                                              GError **error);
+
 void                photos_base_item_set_default_app         (PhotosBaseItem *self, GAppInfo *default_app);
 
 void                photos_base_item_set_default_app_name    (PhotosBaseItem *self, const gchar *default_app_name);
