@@ -42,14 +42,19 @@
 #include "photos-utils.h"
 
 
-struct _PhotosFacebookItemPrivate
+struct _PhotosFacebookItem
 {
+  PhotosBaseItem parent_instance;
   PhotosBaseManager *src_mngr;
 };
 
+struct _PhotosFacebookItemClass
+{
+  PhotosBaseItemClass parent_class;
+};
+
 
 G_DEFINE_TYPE_WITH_CODE (PhotosFacebookItem, photos_facebook_item, PHOTOS_TYPE_BASE_ITEM,
-                         G_ADD_PRIVATE (PhotosFacebookItem)
                          photos_utils_ensure_extension_points ();
                          g_io_extension_point_implement (PHOTOS_BASE_ITEM_EXTENSION_POINT_NAME,
                                                          g_define_type_id,
@@ -67,7 +72,7 @@ photos_facebook_item_create_name_fallback (PhotosBaseItem *item)
   gchar *date_modified_str;
   gint64 mtime;
 
-  provider_name = photos_utils_get_provider_name (self->priv->src_mngr, item);
+  provider_name = photos_utils_get_provider_name (self->src_mngr, item);
 
   mtime = photos_base_item_get_mtime (item);
   date_modified = g_date_time_new_from_unix_local (mtime);
@@ -87,14 +92,14 @@ photos_facebook_item_create_name_fallback (PhotosBaseItem *item)
 static GFBGraphPhoto *
 photos_facebook_get_gfbgraph_photo (PhotosBaseItem *item, GCancellable *cancellable, GError **error)
 {
-  PhotosFacebookItemPrivate *priv = PHOTOS_FACEBOOK_ITEM (item)->priv;
+  PhotosFacebookItem *self = PHOTOS_FACEBOOK_ITEM (item);
   PhotosSource *source;
   const gchar *identifier, *resource_urn;
   GFBGraphGoaAuthorizer *authorizer;
   GFBGraphPhoto *photo = NULL;
 
   resource_urn = photos_base_item_get_resource_urn (item);
-  source = PHOTOS_SOURCE (photos_base_manager_get_object_by_id (priv->src_mngr, resource_urn));
+  source = PHOTOS_SOURCE (photos_base_manager_get_object_by_id (self->src_mngr, resource_urn));
   authorizer = gfbgraph_goa_authorizer_new (photos_source_get_goa_object (source));
   identifier = photos_base_item_get_identifier (item) + strlen ("facebook:");
 
@@ -238,7 +243,7 @@ photos_facebook_item_get_source_widget (PhotosBaseItem *item)
   GtkWidget *source_widget;
   const gchar *name;
 
-  name = photos_utils_get_provider_name (self->priv->src_mngr, item);
+  name = photos_utils_get_provider_name (self->src_mngr, item);
   source_widget = gtk_link_button_new_with_label ("https://www.facebook.com/", name);
   gtk_widget_set_halign (source_widget, GTK_ALIGN_START);
 
@@ -273,7 +278,7 @@ photos_facebook_item_constructed (GObject *object)
 
   G_OBJECT_CLASS (photos_facebook_item_parent_class)->constructed (object);
 
-  name = photos_utils_get_provider_name (self->priv->src_mngr, PHOTOS_BASE_ITEM (self));
+  name = photos_utils_get_provider_name (self->src_mngr, PHOTOS_BASE_ITEM (self));
   photos_base_item_set_default_app_name (PHOTOS_BASE_ITEM (self), name);
 }
 
@@ -283,7 +288,7 @@ photos_facebook_item_dispose (GObject *object)
 {
   PhotosFacebookItem *self = PHOTOS_FACEBOOK_ITEM (object);
 
-  g_clear_object (&self->priv->src_mngr);
+  g_clear_object (&self->src_mngr);
 
   G_OBJECT_CLASS (photos_facebook_item_parent_class)->dispose (object);
 }
@@ -292,17 +297,13 @@ photos_facebook_item_dispose (GObject *object)
 static void
 photos_facebook_item_init (PhotosFacebookItem *self)
 {
-  PhotosFacebookItemPrivate *priv;
   GApplication *app;
   PhotosSearchContextState *state;
 
-  self->priv = photos_facebook_item_get_instance_private (self);
-  priv = self->priv;
-
   app = g_application_get_default ();
   state = photos_search_context_get_state (PHOTOS_SEARCH_CONTEXT (app));
 
-  priv->src_mngr = g_object_ref (state->src_mngr);
+  self->src_mngr = g_object_ref (state->src_mngr);
 }
 
 
