@@ -54,6 +54,14 @@ G_BEGIN_DECLS
   (G_TYPE_CHECK_INSTANCE_TYPE ((obj), \
    PHOTOS_TYPE_MODE_CONTROLLER))
 
+typedef enum
+{
+  PHOTOS_LOAD_STATE_NONE,
+  PHOTOS_LOAD_STATE_ERROR,
+  PHOTOS_LOAD_STATE_FINISHED,
+  PHOTOS_LOAD_STATE_STARTED
+} PhotosLoadState;
+
 typedef enum
 {
   PHOTOS_WINDOW_MODE_NONE,
@@ -87,6 +95,8 @@ PhotosBaseItem           *photos_item_manager_get_active_collection (PhotosItemM
 
 GHashTable               *photos_item_manager_get_collections       (PhotosItemManager *self);
 
+PhotosLoadState           photos_item_manager_get_load_state        (PhotosItemManager *self);
+
 gboolean                  photos_mode_controller_get_can_fullscreen (PhotosModeController *self);
 
 gboolean                  photos_mode_controller_get_fullscreen     (PhotosModeController *self);
