@@ -370,8 +370,8 @@ photos_preview_nav_buttons_dispose (GObject *object)
 
   g_clear_object (&priv->tap_gesture);
   g_clear_object (&priv->model);
-  g_clear_object (&priv->overlay);
-  g_clear_object (&priv->preview_view);
+  g_clear_object (&priv->next_widget);
+  g_clear_object (&priv->prev_widget);
   g_clear_object (&priv->item_mngr);
   g_clear_object (&priv->mode_cntrlr);
 
@@ -402,7 +402,7 @@ photos_preview_nav_buttons_constructed (GObject *object)
 
   G_OBJECT_CLASS (photos_preview_nav_buttons_parent_class)->constructed (object);
 
-  priv->prev_widget = gtk_revealer_new ();
+  priv->prev_widget = g_object_ref_sink (gtk_revealer_new ());
   gtk_widget_set_halign (priv->prev_widget, GTK_ALIGN_START);
   gtk_widget_set_margin_start (priv->prev_widget, 30);
   gtk_widget_set_margin_end (priv->prev_widget, 30);
@@ -428,7 +428,7 @@ photos_preview_nav_buttons_constructed (GObject *object)
                             G_CALLBACK (photos_preview_nav_buttons_leave_notify),
                             self);
 
-  priv->next_widget = gtk_revealer_new ();
+  priv->next_widget = g_object_ref_sink (gtk_revealer_new ());
   gtk_widget_set_halign (priv->next_widget, GTK_ALIGN_END);
   gtk_widget_set_margin_start (priv->next_widget, 30);
   gtk_widget_set_margin_end (priv->next_widget, 30);
@@ -469,6 +469,10 @@ photos_preview_nav_buttons_constructed (GObject *object)
                             "stopped",
                             G_CALLBACK (photos_preview_nav_buttons_multi_press_stopped),
                             self);
+
+  /* We will not need them any more */
+  priv->overlay = NULL;
+  priv->preview_view = NULL;
 }
 
 
@@ -481,11 +485,11 @@ photos_preview_nav_buttons_set_property (GObject *object, guint prop_id, const G
   switch (prop_id)
     {
     case PROP_OVERLAY:
-      priv->overlay = GTK_WIDGET (g_value_dup_object (value));
+      priv->overlay = GTK_WIDGET (g_value_get_object (value)); /* overlay is owned by preview_view */
       break;
 
     case PROP_PREVIEW_VIEW:
-      priv->preview_view = GTK_WIDGET (g_value_dup_object (value));
+      priv->preview_view = GTK_WIDGET (g_value_get_object (value)); /* self is owned by preview_view */
       break;
 
     default:
@@ -566,6 +570,9 @@ photos_preview_nav_buttons_class_init (PhotosPreviewNavButtonsClass *class)
 PhotosPreviewNavButtons *
 photos_preview_nav_buttons_new (PhotosPreviewView *preview_view, GtkOverlay *overlay)
 {
+  g_return_val_if_fail (PHOTOS_IS_PREVIEW_VIEW (preview_view), NULL);
+  g_return_val_if_fail (GTK_IS_OVERLAY (overlay), NULL);
+
   return g_object_new (PHOTOS_TYPE_PREVIEW_NAV_BUTTONS, "preview-view", preview_view, "overlay", overlay, NULL);
 }
 
