@@ -1,6 +1,6 @@
 /*
  * Photos - access, organize and share your photos on GNOME
- * Copyright © 2013, 2014, 2015 Red Hat, Inc.
+ * Copyright © 2013, 2014, 2015, 2016 Red Hat, Inc.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -400,6 +400,22 @@ photos_preview_view_insta (PhotosPreviewView *self, GVariant *parameter)
 }
 
 
+static void
+photos_preview_view_saturation (PhotosPreviewView *self, GVariant *parameter)
+{
+  PhotosBaseItem *item;
+  gdouble scale;
+
+  item = PHOTOS_BASE_ITEM (photos_base_manager_get_active_object (self->priv->item_mngr));
+  if (item == NULL)
+    return;
+
+  scale = g_variant_get_double (parameter);
+  photos_base_item_operation_add (item, "photos:saturation", "scale", scale, NULL);
+  photos_base_item_process_async (item, NULL, photos_preview_view_process, self);
+}
+
+
 static void
 photos_preview_view_sharpen (PhotosPreviewView *self, GVariant *parameter)
 {
@@ -511,6 +527,7 @@ photos_preview_view_dispose (GObject *object)
   g_clear_object (&priv->node);
   g_clear_object (&priv->item_mngr);
   g_clear_object (&priv->mode_cntrlr);
+  g_clear_object (&priv->nav_buttons);
 
   G_OBJECT_CLASS (photos_preview_view_parent_class)->dispose (object);
 }
@@ -640,6 +657,13 @@ photos_preview_view_init (PhotosPreviewView *self)
   action = g_action_map_lookup_action (G_ACTION_MAP (app), "load-previous");
   g_signal_connect_swapped (action, "activate", G_CALLBACK (photos_preview_view_navigate_previous), self);
 
+  action = g_action_map_lookup_action (G_ACTION_MAP (app), "saturation-current");
+  g_signal_connect_object (action,
+                           "activate",
+                           G_CALLBACK (photos_preview_view_saturation),
+                           self,
+                           G_CONNECT_SWAPPED);
+
   action = g_action_map_lookup_action (G_ACTION_MAP (app), "sharpen-current");
   g_signal_connect_object (action, "activate", G_CALLBACK (photos_preview_view_sharpen), self, G_CONNECT_SWAPPED);
 
@@ -713,9 +737,7 @@ photos_preview_view_set_node (PhotosPreviewView *self, GeglNode *node)
 
       priv->node = g_object_ref (node);
       view = photos_preview_view_get_view_from_view_container (view_container);
-
-      /* Steals the reference to the GeglNode. */
-      gegl_gtk_view_set_node (GEGL_GTK_VIEW (view), g_object_ref (priv->node));
+      gegl_gtk_view_set_node (GEGL_GTK_VIEW (view), priv->node);
       gtk_widget_queue_draw (view);
     }
 }
