@@ -115,7 +115,8 @@ photos_search_type_manager_init (PhotosSearchTypeManager *self)
 
   search_type = photos_search_type_new_full (PHOTOS_SEARCH_TYPE_STOCK_FAVORITES,
                                              _("Favorites"),
-                                             "?urn nao:hasTag nao:predefined-tag-favorite",
+                                             "?urn a rdfs:Resource; nao:hasTag nao:predefined-tag-favorite. "
+                                             "OPTIONAL {?item a nie:InformationElement; nie:isPartOf ?urn}",
                                              fav_filter);
   photos_base_manager_add_object (PHOTOS_BASE_MANAGER (self), G_OBJECT (search_type));
   g_object_unref (search_type);
