@@ -1,6 +1,7 @@
 /*
  * Photos - access, organize and share your photos on GNOME
  * Copyright © 2015 Red Hat, Inc.
+ * Copyright © 2015 Umang Jain
  * Copyright © 2011, 2012, 2013, 2014, 2015 Yorba Foundation
  *
  * This program is free software; you can redistribute it and/or
@@ -59,12 +60,14 @@ struct _PhotosToolCrop
 {
   PhotosTool parent_instance;
   GAction *crop;
-  GeglRectangle bbox_scaled;
+  GeglRectangle bbox_zoomed;
   GeglRectangle bbox_source;
   GtkListStore *model;
+  GtkWidget *box;
   GtkWidget *combo_box;
-  GtkWidget *grid;
+  GtkWidget *lock_button;
   GtkWidget *reset_button;
+  GtkWidget *revealer;
   GtkWidget *view;
   PhotosToolCropLocation location;
   cairo_surface_t *surface;
@@ -121,6 +124,9 @@ enum
   CONSTRAINT_COLUMN_BASIS_WIDTH = 3
 };
 
+/* "Free" is excluded from the GtkComboBox and represented by the
+ *  GtkCheckButton. Adjust accordingly.
+ */
 static PhotosToolCropConstraint CONSTRAINTS[] =
 {
   { PHOTOS_TOOL_CROP_ASPECT_RATIO_ANY, N_("Free"), 0, 0 },
@@ -229,18 +235,18 @@ static void
 photos_tool_crop_surface_create (PhotosToolCrop *self)
 {
   GdkWindow *window;
-  gfloat scale;
+  gfloat zoom;
 
   g_clear_pointer (&self->surface, (GDestroyNotify) cairo_surface_destroy);
 
   window = gtk_widget_get_window (self->view);
-  scale = gegl_gtk_view_get_scale (GEGL_GTK_VIEW (self->view));
-  self->bbox_scaled.height = (gint) (scale * self->bbox_source.height + 0.5);
-  self->bbox_scaled.width = (gint) (scale * self->bbox_source.width + 0.5);
+  zoom = gegl_gtk_view_get_zoom (GEGL_GTK_VIEW (self->view));
+  self->bbox_zoomed.height = (gint) (zoom * self->bbox_source.height + 0.5);
+  self->bbox_zoomed.width = (gint) (zoom * self->bbox_source.width + 0.5);
   self->surface = gdk_window_create_similar_surface (window,
                                                      CAIRO_CONTENT_COLOR_ALPHA,
-                                                     self->bbox_scaled.width,
-                                                     self->bbox_scaled.height);
+                                                     self->bbox_zoomed.width,
+                                                     self->bbox_zoomed.height);
 }
 
 
@@ -300,13 +306,30 @@ photos_tool_crop_change_constraint (PhotosToolCrop *self)
 }
 
 
+static gint
+photos_tool_crop_get_active (PhotosToolCrop *self)
+{
+  gint active;
+
+  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (self->lock_button)))
+    {
+      active = gtk_combo_box_get_active (GTK_COMBO_BOX (self->combo_box)) + 1;
+      g_return_val_if_fail (active >= 1, -1);
+    }
+  else
+    active = 0;
+
+  return active;
+}
+
+
 static void
 photos_tool_crop_init_crop (PhotosToolCrop *self)
 {
   gdouble aspect_ratio;
   gint active;
 
-  active = gtk_combo_box_get_active (GTK_COMBO_BOX (self->combo_box));
+  active = photos_tool_crop_get_active (self);
   g_return_if_fail (active >= 0);
 
   self->crop_aspect_ratio = photos_tool_crop_calculate_aspect_ratio (self, (guint) active);
@@ -317,17 +340,17 @@ photos_tool_crop_init_crop (PhotosToolCrop *self)
 
   if (self->crop_aspect_ratio < aspect_ratio)
     {
-      self->crop_height = 0.7 * self->bbox_scaled.height;
+      self->crop_height = 0.7 * self->bbox_zoomed.height;
       self->crop_width = self->crop_height * self->crop_aspect_ratio;
     }
   else
     {
-      self->crop_width = 0.7 * self->bbox_scaled.width;
+      self->crop_width = 0.7 * self->bbox_zoomed.width;
       self->crop_height = self->crop_width / self->crop_aspect_ratio;
     }
 
-  self->crop_x = ((gdouble) self->bbox_scaled.width - self->crop_width) / 2.0;
-  self->crop_y = ((gdouble) self->bbox_scaled.height - self->crop_height) / 2.0;
+  self->crop_x = ((gdouble) self->bbox_zoomed.width - self->crop_width) / 2.0;
+  self->crop_y = ((gdouble) self->bbox_zoomed.height - self->crop_height) / 2.0;
 
  out:
   photos_tool_crop_surface_draw (self);
@@ -555,8 +578,8 @@ photos_tool_crop_set_crop (PhotosToolCrop *self, gdouble event_x, gdouble event_
       {
         gboolean x_adj = FALSE;
         gboolean y_adj = FALSE;
-        gdouble bbox_scaled_height = (gdouble) self->bbox_scaled.height;
-        gdouble bbox_scaled_width = (gdouble) self->bbox_scaled.width;
+        gdouble bbox_zoomed_height = (gdouble) self->bbox_zoomed.height;
+        gdouble bbox_zoomed_width = (gdouble) self->bbox_zoomed.width;
 
         self->crop_x += delta_x;
         self->crop_y += delta_y;
@@ -575,15 +598,15 @@ photos_tool_crop_set_crop (PhotosToolCrop *self, gdouble event_x, gdouble event_
             y_adj = TRUE;
           }
 
-        if (self->crop_x + self->crop_width > bbox_scaled_width)
+        if (self->crop_x + self->crop_width > bbox_zoomed_width)
           {
-            self->crop_width = bbox_scaled_width - self->crop_x;
+            self->crop_width = bbox_zoomed_width - self->crop_x;
             x_adj = TRUE;
           }
 
-        if (self->crop_y + self->crop_height > bbox_scaled_height)
+        if (self->crop_y + self->crop_height > bbox_zoomed_height)
           {
-            self->crop_height = bbox_scaled_height - self->crop_y;
+            self->crop_height = bbox_zoomed_height - self->crop_y;
             y_adj = TRUE;
           }
 
@@ -593,7 +616,7 @@ photos_tool_crop_set_crop (PhotosToolCrop *self, gdouble event_x, gdouble event_
               self->crop_width = crop_width_old;
             else
               {
-                self->crop_x = bbox_scaled_width - crop_width_old;
+                self->crop_x = bbox_zoomed_width - crop_width_old;
                 self->crop_width = crop_width_old;
               }
           }
@@ -604,7 +627,7 @@ photos_tool_crop_set_crop (PhotosToolCrop *self, gdouble event_x, gdouble event_
               self->crop_height = crop_height_old;
             else
               {
-                self->crop_y = bbox_scaled_height - crop_height_old;
+                self->crop_y = bbox_zoomed_height - crop_height_old;
                 self->crop_height = crop_height_old;
               }
           }
@@ -649,9 +672,9 @@ photos_tool_crop_set_crop (PhotosToolCrop *self, gdouble event_x, gdouble event_
   if (self->crop_height < CROP_MIN_SIZE
       || self->crop_width < CROP_MIN_SIZE
       || self->crop_x < 0.0
-      || self->crop_x + self->crop_width > self->bbox_scaled.width
+      || self->crop_x + self->crop_width > self->bbox_zoomed.width
       || self->crop_y < 0.0
-      || self->crop_y + self->crop_height > self->bbox_scaled.height)
+      || self->crop_y + self->crop_height > self->bbox_zoomed.height)
     {
       self->crop_height = crop_height_old;
       self->crop_width = crop_width_old;
@@ -770,11 +793,13 @@ photos_tool_crop_set_location (PhotosToolCrop *self, gdouble event_x, gdouble ev
 
 
 static void
-photos_tool_crop_changed (PhotosToolCrop *self)
+photos_tool_crop_active_changed (PhotosToolCrop *self)
 {
   gint active;
 
-  active = gtk_combo_box_get_active (GTK_COMBO_BOX (self->combo_box));
+  g_return_if_fail (!self->reset);
+
+  active = photos_tool_crop_get_active (self);
   g_return_if_fail (active >= 0);
 
   self->crop_aspect_ratio = photos_tool_crop_calculate_aspect_ratio (self, (guint) active);
@@ -785,13 +810,43 @@ photos_tool_crop_changed (PhotosToolCrop *self)
 }
 
 
+static void
+photos_tool_crop_set_active (PhotosToolCrop *self, gint active)
+{
+  g_signal_handlers_block_by_func (self->combo_box, photos_tool_crop_active_changed, self);
+  g_signal_handlers_block_by_func (self->lock_button, photos_tool_crop_active_changed, self);
+
+  if (active == -1) /* reset */
+    {
+      gtk_combo_box_set_active (GTK_COMBO_BOX (self->combo_box), 0);
+      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (self->lock_button), TRUE);
+    }
+  else if (active == 0)
+    {
+      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (self->lock_button), FALSE);
+    }
+  else if (active > 0)
+    {
+      gtk_combo_box_set_active (GTK_COMBO_BOX (self->combo_box), active);
+      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (self->lock_button), TRUE);
+    }
+  else
+    {
+      g_warn_if_reached ();
+    }
+
+  g_signal_handlers_unblock_by_func (self->combo_box, photos_tool_crop_active_changed, self);
+  g_signal_handlers_unblock_by_func (self->lock_button, photos_tool_crop_active_changed, self);
+}
+
+
 static void
 photos_tool_crop_process (GObject *source_object, GAsyncResult *res, gpointer user_data)
 {
   PhotosToolCrop *self = PHOTOS_TOOL_CROP (user_data);
   GError *error = NULL;
   PhotosBaseItem *item = PHOTOS_BASE_ITEM (source_object);
-  gfloat scale;
+  gfloat zoom;
   guint active;
 
   photos_base_item_process_finish (item, res, &error);
@@ -802,18 +857,15 @@ photos_tool_crop_process (GObject *source_object, GAsyncResult *res, gpointer us
       goto out;
     }
 
-  scale = gegl_gtk_view_get_scale (GEGL_GTK_VIEW (self->view));
-  self->crop_height *= scale;
-  self->crop_width *= scale;
-  self->crop_x *= scale;
-  self->crop_y *= scale;
+  zoom = gegl_gtk_view_get_zoom (GEGL_GTK_VIEW (self->view));
+  self->crop_height *= zoom;
+  self->crop_width *= zoom;
+  self->crop_x *= zoom;
+  self->crop_y *= zoom;
 
   self->crop_aspect_ratio = self->crop_width / self->crop_height;
   active = photos_tool_crop_find_constraint (self, self->crop_aspect_ratio);
-
-  g_signal_handlers_block_by_func (self->combo_box, photos_tool_crop_changed, self);
-  gtk_combo_box_set_active (GTK_COMBO_BOX (self->combo_box), (gint) active);
-  g_signal_handlers_unblock_by_func (self->combo_box, photos_tool_crop_changed, self);
+  photos_tool_crop_set_active (self, (gint) active);
 
   photos_tool_crop_surface_draw (self);
 
@@ -827,6 +879,7 @@ static void
 photos_tool_crop_reset_clicked (PhotosToolCrop *self)
 {
   self->reset = TRUE;
+  photos_tool_crop_set_active (self, -1);
   g_signal_emit_by_name (self, "hide-requested");
 }
 
@@ -839,17 +892,17 @@ photos_tool_crop_size_allocate (PhotosToolCrop *self, GdkRectangle *allocation)
   gdouble crop_x_ratio;
   gdouble crop_y_ratio;
 
-  crop_height_ratio = self->crop_height / (gdouble) self->bbox_scaled.height;
-  crop_width_ratio = self->crop_width / (gdouble) self->bbox_scaled.width;
-  crop_x_ratio = self->crop_x / (gdouble) self->bbox_scaled.width;
-  crop_y_ratio = self->crop_y / (gdouble) self->bbox_scaled.height;
+  crop_height_ratio = self->crop_height / (gdouble) self->bbox_zoomed.height;
+  crop_width_ratio = self->crop_width / (gdouble) self->bbox_zoomed.width;
+  crop_x_ratio = self->crop_x / (gdouble) self->bbox_zoomed.width;
+  crop_y_ratio = self->crop_y / (gdouble) self->bbox_zoomed.height;
 
   photos_tool_crop_surface_create (self);
 
-  self->crop_height = crop_height_ratio * (gdouble) self->bbox_scaled.height;
-  self->crop_width = crop_width_ratio * (gdouble) self->bbox_scaled.width;
-  self->crop_x = crop_x_ratio * (gdouble) self->bbox_scaled.width;
-  self->crop_y = crop_y_ratio * (gdouble) self->bbox_scaled.height;
+  self->crop_height = crop_height_ratio * (gdouble) self->bbox_zoomed.height;
+  self->crop_width = crop_width_ratio * (gdouble) self->bbox_zoomed.width;
+  self->crop_x = crop_x_ratio * (gdouble) self->bbox_zoomed.width;
+  self->crop_y = crop_y_ratio * (gdouble) self->bbox_zoomed.height;
 
   photos_tool_crop_surface_draw (self);
 }
@@ -885,7 +938,7 @@ photos_tool_crop_activate (PhotosTool *tool, PhotosBaseItem *item, GeglGtkView *
       g_return_if_fail (y >= 0.0);
 
       /* These values are invalid until they are multiplied by the
-       * view's scale, which we won't know until we have finished
+       * view's zoom, which we won't know until we have finished
        * processing with the reset gegl:crop values.
        */
       self->crop_height = height;
@@ -929,16 +982,22 @@ photos_tool_crop_deactivate (PhotosTool *tool)
     {
       GVariantBuilder parameter;
       GVariantType *parameter_type;
-      gfloat scale;
+      gfloat zoom;
 
-      scale = gegl_gtk_view_get_scale (GEGL_GTK_VIEW (self->view));
+      zoom = gegl_gtk_view_get_zoom (GEGL_GTK_VIEW (self->view));
 
+      /* GEGL needs the parameters to be in device pixels. So, we
+       * should multiply the extents of the crop rectangle and the
+       * zoom by the scale factor to convert them. However, the scale
+       * factor will cancel itself in the numerator and denominator,
+       * so, in practice, the conversion is unnecessary.
+       */
       parameter_type = g_variant_type_new ("a{sd}");
       g_variant_builder_init (&parameter, parameter_type);
-      g_variant_builder_add (&parameter, "{sd}", "height", self->crop_height / scale);
-      g_variant_builder_add (&parameter, "{sd}", "width", self->crop_width / scale);
-      g_variant_builder_add (&parameter, "{sd}", "x", self->crop_x / scale);
-      g_variant_builder_add (&parameter, "{sd}", "y", self->crop_y / scale);
+      g_variant_builder_add (&parameter, "{sd}", "height", self->crop_height / zoom);
+      g_variant_builder_add (&parameter, "{sd}", "width", self->crop_width / zoom);
+      g_variant_builder_add (&parameter, "{sd}", "x", self->crop_x / zoom);
+      g_variant_builder_add (&parameter, "{sd}", "y", self->crop_y / zoom);
       g_action_activate (self->crop, g_variant_builder_end (&parameter));
 
       g_variant_type_free (parameter_type);
@@ -982,7 +1041,7 @@ static GtkWidget *
 photos_tool_crop_get_widget (PhotosTool *tool)
 {
   PhotosToolCrop *self = PHOTOS_TOOL_CROP (tool);
-  return self->grid;
+  return self->box;
 }
 
 
@@ -1061,7 +1120,7 @@ photos_tool_crop_dispose (GObject *object)
   PhotosToolCrop *self = PHOTOS_TOOL_CROP (object);
 
   g_clear_object (&self->model);
-  g_clear_object (&self->grid);
+  g_clear_object (&self->box);
   g_clear_pointer (&self->surface, (GDestroyNotify) cairo_surface_destroy);
 
   G_OBJECT_CLASS (photos_tool_crop_parent_class)->dispose (object);
@@ -1080,7 +1139,7 @@ photos_tool_crop_init (PhotosToolCrop *self)
 
   self->model = gtk_list_store_new (4, G_TYPE_INT, G_TYPE_STRING, G_TYPE_UINT, G_TYPE_UINT);
 
-  for (i = 0; i < G_N_ELEMENTS (CONSTRAINTS); i++)
+  for (i = 1; i < G_N_ELEMENTS (CONSTRAINTS); i++)
     {
       GtkTreeIter iter;
 
@@ -1094,18 +1153,30 @@ photos_tool_crop_init (PhotosToolCrop *self)
                           -1);
     }
 
-  self->grid = g_object_ref_sink (gtk_grid_new ());
-  gtk_orientable_set_orientation (GTK_ORIENTABLE (self->grid), GTK_ORIENTATION_VERTICAL);
-  gtk_grid_set_row_spacing (GTK_GRID (self->grid), 12);
+  /* We really need a GtkBox here. A GtkGrid won't work because it
+   * doesn't expand the children to fill the full width of the
+   * palette.
+   */
+  self->box = g_object_ref_sink (gtk_box_new (GTK_ORIENTATION_VERTICAL, 12));
+
+  self->lock_button = gtk_check_button_new_with_label (_("Lock aspect ratio"));
+  gtk_container_add (GTK_CONTAINER (self->box), self->lock_button);
+  g_signal_connect_swapped (self->lock_button, "toggled", G_CALLBACK (photos_tool_crop_active_changed), self);
+
+  self->revealer = gtk_revealer_new ();
+  gtk_container_add (GTK_CONTAINER (self->box), self->revealer);
+  gtk_revealer_set_transition_type (GTK_REVEALER (self->revealer), GTK_REVEALER_TRANSITION_TYPE_SLIDE_DOWN);
+  g_object_bind_property (self->lock_button, "active", self->revealer, "reveal-child", G_BINDING_DEFAULT);
 
   self->combo_box = gtk_combo_box_new_with_model (GTK_TREE_MODEL (self->model));
-  gtk_combo_box_set_active (GTK_COMBO_BOX (self->combo_box), 1);
-  gtk_container_add (GTK_CONTAINER (self->grid), self->combo_box);
-  g_signal_connect_swapped (self->combo_box, "changed", G_CALLBACK (photos_tool_crop_changed), self);
+  gtk_container_add (GTK_CONTAINER (self->revealer), self->combo_box);
+  g_signal_connect_swapped (self->combo_box, "changed", G_CALLBACK (photos_tool_crop_active_changed), self);
+
+  photos_tool_crop_set_active (self, -1);
 
   self->reset_button = gtk_button_new_with_label (_("Reset"));
   gtk_widget_set_halign (self->reset_button, GTK_ALIGN_END);
-  gtk_container_add (GTK_CONTAINER (self->grid), self->reset_button);
+  gtk_container_add (GTK_CONTAINER (self->box), self->reset_button);
   g_signal_connect_swapped (self->reset_button, "clicked", G_CALLBACK (photos_tool_crop_reset_clicked), self);
 
   renderer = gtk_cell_renderer_text_new ();
