@@ -133,13 +133,15 @@ photos_tracker_controller_cursor_next (GObject *source_object, GAsyncResult *res
   gboolean valid;
   gint64 now;
 
+  if (priv->item_mngr == NULL)
+    goto out;
+
   valid = tracker_sparql_cursor_next_finish (cursor, res, NULL); /* TODO: use GError */
   if (!valid)
     {
       tracker_sparql_cursor_close (cursor);
       photos_tracker_controller_query_finished (self, NULL);
-      g_object_unref (self);
-      return;
+      goto out;
     }
 
   now = g_get_monotonic_time ();
@@ -149,7 +151,10 @@ photos_tracker_controller_cursor_next (GObject *source_object, GAsyncResult *res
   tracker_sparql_cursor_next_async (cursor,
                                     priv->cancellable,
                                     photos_tracker_controller_cursor_next,
-                                    self);
+                                    g_object_ref (self));
+
+ out:
+  g_object_unref (self);
 }
 
 
@@ -187,6 +192,8 @@ photos_tracker_controller_perform_current_query (PhotosTrackerController *self)
     photos_query_free (priv->current_query);
 
   priv->current_query = PHOTOS_TRACKER_CONTROLLER_GET_CLASS (self)->get_query (self);
+  g_return_if_fail (priv->current_query != NULL);
+
   g_cancellable_reset (priv->cancellable);
 
   if (G_UNLIKELY (priv->queue == NULL))
@@ -288,6 +295,8 @@ photos_tracker_controller_source_object_added_removed (PhotosTrackerController *
   PhotosTrackerControllerPrivate *priv = self->priv;
   PhotosWindowMode mode;
 
+  g_return_if_fail (priv->mode_cntrlr != NULL);
+
   mode = photos_mode_controller_get_window_mode (priv->mode_cntrlr);
   if (mode == priv->mode)
     photos_tracker_controller_refresh_for_source (self);
@@ -330,9 +339,7 @@ photos_tracker_controller_dispose (GObject *object)
   PhotosTrackerController *self = PHOTOS_TRACKER_CONTROLLER (object);
   PhotosTrackerControllerPrivate *priv = self->priv;
 
-  g_clear_object (&priv->item_mngr);
   g_clear_object (&priv->src_mngr);
-  g_clear_object (&priv->mode_cntrlr);
   g_clear_object (&priv->offset_cntrlr);
   g_clear_object (&priv->queue);
 
@@ -346,6 +353,12 @@ photos_tracker_controller_finalize (GObject *object)
   PhotosTrackerController *self = PHOTOS_TRACKER_CONTROLLER (object);
   PhotosTrackerControllerPrivate *priv = self->priv;
 
+  if (priv->item_mngr != NULL)
+    g_object_remove_weak_pointer (G_OBJECT (priv->item_mngr), (gpointer *) &priv->item_mngr);
+
+  if (priv->mode_cntrlr != NULL)
+    g_object_remove_weak_pointer (G_OBJECT (priv->mode_cntrlr), (gpointer *) &priv->mode_cntrlr);
+
   g_clear_error (&priv->queue_error);
 
   if (priv->current_query != NULL)
@@ -387,7 +400,9 @@ photos_tracker_controller_init (PhotosTrackerController *self)
   state = photos_search_context_get_state (PHOTOS_SEARCH_CONTEXT (app));
 
   priv->cancellable = g_cancellable_new ();
-  priv->item_mngr = g_object_ref (state->item_mngr);
+
+  priv->item_mngr = state->item_mngr;
+  g_object_add_weak_pointer (G_OBJECT (priv->item_mngr), (gpointer *) &priv->item_mngr);
 
   priv->src_mngr = g_object_ref (state->src_mngr);
   g_signal_connect_swapped (priv->src_mngr,
@@ -399,7 +414,8 @@ photos_tracker_controller_init (PhotosTrackerController *self)
                             G_CALLBACK (photos_tracker_controller_source_object_added_removed),
                             self);
 
-  priv->mode_cntrlr = g_object_ref (state->mode_cntrlr);
+  priv->mode_cntrlr = state->mode_cntrlr;
+  g_object_add_weak_pointer (G_OBJECT (priv->mode_cntrlr), (gpointer *) &priv->mode_cntrlr);
   g_signal_connect_swapped (priv->mode_cntrlr,
                             "window-mode-changed",
                             G_CALLBACK (photos_tracker_controller_window_mode_changed),
