@@ -1,6 +1,6 @@
 /*
  * Photos - access, organize and share your photos on GNOME
- * Copyright © 2012, 2013, 2014, 2015 Red Hat, Inc.
+ * Copyright © 2012, 2013, 2014, 2015, 2016 Red Hat, Inc.
  * Copyright © 2009 Yorba Foundation
  *
  * This program is free software; you can redistribute it and/or
@@ -44,6 +44,9 @@
 #include "photos-media-server-item.h"
 #include "photos-operation-insta-curve.h"
 #include "photos-operation-insta-filter.h"
+#include "photos-operation-jpg-guess-sizes.h"
+#include "photos-operation-png-guess-sizes.h"
+#include "photos-operation-saturation.h"
 #include "photos-query.h"
 #include "photos-source.h"
 #include "photos-tool.h"
@@ -107,6 +110,103 @@ photos_utils_border_pixbuf (GdkPixbuf *pixbuf)
 }
 
 
+static GeglBuffer *
+photos_utils_buffer_zoom (GeglBuffer *buffer, gdouble zoom, GCancellable *cancellable, GError **error)
+{
+  GeglBuffer *ret_val = NULL;
+  GeglNode *buffer_sink;
+  GeglNode *buffer_source;
+  GeglNode *graph;
+  GeglNode *scale;
+
+  graph = gegl_node_new ();
+  buffer_source = gegl_node_new_child (graph, "operation", "gegl:buffer-source", "buffer", buffer, NULL);
+  scale = gegl_node_new_child (graph, "operation", "gegl:scale-ratio", "x", zoom, "y", zoom, NULL);
+  buffer_sink = gegl_node_new_child (graph, "operation", "gegl:buffer-sink", "buffer", &ret_val, NULL);
+  gegl_node_link_many (buffer_source, scale, buffer_sink, NULL);
+  gegl_node_process (buffer_sink);
+
+  g_object_unref (graph);
+  return ret_val;
+}
+
+
+static void
+photos_utils_buffer_zoom_in_thread_func (GTask *task,
+                                         gpointer source_object,
+                                         gpointer task_data,
+                                         GCancellable *cancellable)
+{
+  GeglBuffer *buffer = GEGL_BUFFER (source_object);
+  GeglBuffer *result = NULL;
+  GError *error;
+  const gchar *zoom_str = (const gchar *) task_data;
+  gchar *endptr;
+  gdouble zoom;
+
+  zoom = g_ascii_strtod (zoom_str, &endptr);
+  g_assert (*endptr == '\0');
+
+  error = NULL;
+  result = photos_utils_buffer_zoom (buffer, zoom, cancellable, &error);
+  if (error != NULL)
+    {
+      g_task_return_error (task, error);
+      goto out;
+    }
+
+  g_task_return_pointer (task, g_object_ref (result), g_object_unref);
+
+ out:
+  g_clear_object (&result);
+}
+
+
+void
+photos_utils_buffer_zoom_async (GeglBuffer *buffer,
+                                gdouble zoom,
+                                GCancellable *cancellable,
+                                GAsyncReadyCallback callback,
+                                gpointer user_data)
+{
+  GTask *task;
+  gchar zoom_str[G_ASCII_DTOSTR_BUF_SIZE];
+
+  g_return_if_fail (GEGL_IS_BUFFER (buffer));
+  g_return_if_fail (zoom > 0.0);
+
+  task = g_task_new (buffer, cancellable, callback, user_data);
+  g_task_set_source_tag (task, photos_utils_buffer_zoom_async);
+
+  if (GEGL_FLOAT_EQUAL ((gfloat) zoom, 1.0))
+    {
+      g_task_return_pointer (task, g_object_ref (buffer), g_object_unref);
+      goto out;
+    }
+
+  g_ascii_dtostr (zoom_str, G_N_ELEMENTS (zoom_str), zoom);
+  g_task_set_task_data (task, g_strdup (zoom_str), g_free);
+
+  g_task_run_in_thread (task, photos_utils_buffer_zoom_in_thread_func);
+
+ out:
+  g_object_unref (task);
+}
+
+
+GeglBuffer *
+photos_utils_buffer_zoom_finish (GeglBuffer *buffer, GAsyncResult *res, GError **error)
+{
+  GTask *task = G_TASK (res);
+
+  g_return_val_if_fail (g_task_is_valid (res, buffer), NULL);
+  g_return_val_if_fail (g_task_get_source_tag (task) == photos_utils_buffer_zoom_async, NULL);
+  g_return_val_if_fail (error == NULL || *error == NULL, NULL);
+
+  return g_task_propagate_pointer (task, error);
+}
+
+
 GdkPixbuf *
 photos_utils_center_pixbuf (GdkPixbuf *pixbuf, gint size)
 {
@@ -142,6 +242,26 @@ photos_utils_convert_path_to_uri (const gchar *path)
 }
 
 
+GeglBuffer *
+photos_utils_create_buffer_from_node (GeglNode *node)
+{
+  GeglBuffer *buffer = NULL;
+  GeglNode *buffer_sink;
+  GeglNode *graph;
+
+  graph = gegl_node_get_parent (node);
+  buffer_sink = gegl_node_new_child (graph,
+                                     "operation", "gegl:buffer-sink",
+                                     "buffer", &buffer,
+                                     NULL);
+  gegl_node_link (node, buffer_sink);
+  gegl_node_process (buffer_sink);
+  g_object_unref (buffer_sink);
+
+  return buffer;
+}
+
+
 GIcon *
 photos_utils_create_collection_icon (gint base_size, GList *pixbufs)
 {
@@ -693,6 +813,9 @@ photos_utils_ensure_builtins (void)
 
       g_type_ensure (PHOTOS_TYPE_OPERATION_INSTA_CURVE);
       g_type_ensure (PHOTOS_TYPE_OPERATION_INSTA_FILTER);
+      g_type_ensure (PHOTOS_TYPE_OPERATION_JPG_GUESS_SIZES);
+      g_type_ensure (PHOTOS_TYPE_OPERATION_PNG_GUESS_SIZES);
+      g_type_ensure (PHOTOS_TYPE_OPERATION_SATURATION);
 
       g_type_ensure (PHOTOS_TYPE_TOOL_COLORS);
       g_type_ensure (PHOTOS_TYPE_TOOL_CROP);
@@ -916,41 +1039,6 @@ photos_utils_get_pixbuf_common_suffix (GdkPixbufFormat *format)
 }
 
 
-gchar *
-photos_utils_get_pixbuf_type_from_mime_type (const gchar *mime_type)
-{
-  GSList *formats;
-  GSList *l;
-  gchar *ret_val = NULL;
-
-  formats = gdk_pixbuf_get_formats ();
-
-  for (l = formats; l != NULL; l = l->next)
-    {
-      GdkPixbufFormat *format = (GdkPixbufFormat*) l->data;
-      gchar **supported_mime_types;
-      guint i;
-
-      supported_mime_types = gdk_pixbuf_format_get_mime_types (format);
-      for (i = 0; supported_mime_types[i] != NULL; i++)
-        {
-          if (g_strcmp0 (mime_type, supported_mime_types[i]) == 0)
-            {
-              ret_val = gdk_pixbuf_format_get_name (format);
-              break;
-            }
-        }
-
-      g_strfreev (supported_mime_types);
-      if (ret_val != NULL)
-        break;
-    }
-
-  g_slist_free (formats);
-  return ret_val;
-}
-
-
 const gchar *
 photos_utils_get_provider_name (PhotosBaseManager *src_mngr, PhotosBaseItem *item)
 {
@@ -1017,6 +1105,50 @@ photos_utils_icon_from_rdf_type (const gchar *type)
 }
 
 
+void
+photos_utils_list_box_header_func (GtkListBoxRow *row, GtkListBoxRow *before, gpointer user_data)
+{
+  GtkWidget *header;
+
+  if (before == NULL)
+    {
+      gtk_list_box_row_set_header (row, NULL);
+      return;
+    }
+
+  header = gtk_list_box_row_get_header (row);
+  if (header == NULL)
+    {
+      header = gtk_separator_new (GTK_ORIENTATION_HORIZONTAL);
+      gtk_widget_show (header);
+      gtk_list_box_row_set_header (row, header);
+    }
+}
+
+
+gboolean
+photos_utils_make_directory_with_parents (GFile *file, GCancellable *cancellable, GError **error)
+{
+  GError *local_error = NULL;
+  gboolean ret_val;
+
+  ret_val = g_file_make_directory_with_parents (file, cancellable, &local_error);
+  if (local_error != NULL)
+    {
+      if (g_error_matches (local_error, G_IO_ERROR, G_IO_ERROR_EXISTS))
+        {
+          g_clear_error (&local_error);
+          ret_val = TRUE;
+        }
+    }
+
+  if (local_error != NULL)
+    g_propagate_error (error, local_error);
+
+  return ret_val;
+}
+
+
 GQuark
 photos_utils_orientation_bottom_quark (void)
 {
