@@ -1,6 +1,6 @@
 /*
  * Photos - access, organize and share your photos on GNOME
- * Copyright © 2012, 2013, 2014, 2015 Red Hat, Inc.
+ * Copyright © 2012, 2013, 2014, 2015, 2016 Red Hat, Inc.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -55,13 +55,24 @@ G_BEGIN_DECLS
 
 #define PHOTOS_COLLECTION_SCREENSHOT \
   "http://www.semanticdesktop.org/ontologies/2007/03/22/nfo#image-category-screenshot"
+#define PHOTOS_EXPORT_SUBPATH "Exports"
 
 void             photos_utils_border_pixbuf               (GdkPixbuf *pixbuf);
 
+void             photos_utils_buffer_zoom_async           (GeglBuffer *buffer,
+                                                           gdouble zoom,
+                                                           GCancellable *cancellable,
+                                                           GAsyncReadyCallback callback,
+                                                           gpointer user_data);
+
+GeglBuffer      *photos_utils_buffer_zoom_finish          (GeglBuffer *buffer, GAsyncResult *res, GError **error);
+
 GdkPixbuf       *photos_utils_center_pixbuf               (GdkPixbuf *pixbuf, gint size);
 
 gchar           *photos_utils_convert_path_to_uri         (const gchar *path);
 
+GeglBuffer      *photos_utils_create_buffer_from_node     (GeglNode *node);
+
 GIcon           *photos_utils_create_collection_icon      (gint base_size, GList *pixbufs);
 
 GeglNode        *photos_utils_create_orientation_node     (GeglNode *parent, GQuark orientation);
@@ -120,8 +131,6 @@ gint             photos_utils_get_icon_size_unscaled      (void);
 
 char*            photos_utils_get_pixbuf_common_suffix    (GdkPixbufFormat *format);
 
-gchar           *photos_utils_get_pixbuf_type_from_mime_type (const gchar *mime_type);
-
 const gchar     *photos_utils_get_provider_name           (PhotosBaseManager *src_mngr, PhotosBaseItem *item);
 
 GtkBorder       *photos_utils_get_thumbnail_frame_border  (void);
@@ -130,6 +139,12 @@ GList           *photos_utils_get_urns_from_paths         (GList *paths, GtkTree
 
 GIcon           *photos_utils_icon_from_rdf_type          (const gchar *type);
 
+void             photos_utils_list_box_header_func        (GtkListBoxRow *row,
+                                                           GtkListBoxRow *before,
+                                                           gpointer user_data);
+
+gboolean         photos_utils_make_directory_with_parents (GFile *file, GCancellable *cancellable, GError **error);
+
 GQuark           photos_utils_orientation_bottom_quark    (void);
 
 GQuark           photos_utils_orientation_left_quark      (void);
